"""Wrapper for http://oss.sgi.com/projects/ogl-sample/ABI/wglext.h

Generated by tools/gengl.py.
Do not modify this file.
"""
from __future__ import annotations

from ctypes import CFUNCTYPE, POINTER, Structure, c_char_p, c_float, c_int, c_long, c_ubyte, c_uint, c_ulong, c_ushort
from ctypes.wintypes import HANDLE, CHAR, DWORD, LONG, HDC, UINT, USHORT, LPVOID, FLOAT

from pyglet.gl.lib import c_void
from pyglet.gl.lib import link_WGL as _link_function
from pyglet.libs.win32.types import INT64, INT32

# BEGIN GENERATED CONTENT (do not edit below this line)

# This content is generated by tools/gengl.py.
# Wrapper for http://www.opengl.org/registry/api/wglext.h


# H (C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:7)
# H (C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:7)
WIN32_LEAN_AND_MEAN = 1 	# http://www.opengl.org/registry/api/wglext.h:34
WGL_WGLEXT_VERSION = 11 	# http://www.opengl.org/registry/api/wglext.h:53
# ARB_buffer_region (http://www.opengl.org/registry/api/wglext.h:55)
WGL_FRONT_COLOR_BUFFER_BIT_ARB = 1 	# http://www.opengl.org/registry/api/wglext.h:56
WGL_BACK_COLOR_BUFFER_BIT_ARB = 2 	# http://www.opengl.org/registry/api/wglext.h:57
WGL_DEPTH_BUFFER_BIT_ARB = 4 	# http://www.opengl.org/registry/api/wglext.h:58
WGL_STENCIL_BUFFER_BIT_ARB = 8 	# http://www.opengl.org/registry/api/wglext.h:59
# ARB_multisample (http://www.opengl.org/registry/api/wglext.h:62)
WGL_SAMPLE_BUFFERS_ARB = 8257 	# http://www.opengl.org/registry/api/wglext.h:63
WGL_SAMPLES_ARB = 8258 	# http://www.opengl.org/registry/api/wglext.h:64
# ARB_extensions_string (http://www.opengl.org/registry/api/wglext.h:67)
# ARB_pixel_format (http://www.opengl.org/registry/api/wglext.h:70)
WGL_NUMBER_PIXEL_FORMATS_ARB = 8192 	# http://www.opengl.org/registry/api/wglext.h:71
WGL_DRAW_TO_WINDOW_ARB = 8193 	# http://www.opengl.org/registry/api/wglext.h:72
WGL_DRAW_TO_BITMAP_ARB = 8194 	# http://www.opengl.org/registry/api/wglext.h:73
WGL_ACCELERATION_ARB = 8195 	# http://www.opengl.org/registry/api/wglext.h:74
WGL_NEED_PALETTE_ARB = 8196 	# http://www.opengl.org/registry/api/wglext.h:75
WGL_NEED_SYSTEM_PALETTE_ARB = 8197 	# http://www.opengl.org/registry/api/wglext.h:76
WGL_SWAP_LAYER_BUFFERS_ARB = 8198 	# http://www.opengl.org/registry/api/wglext.h:77
WGL_SWAP_METHOD_ARB = 8199 	# http://www.opengl.org/registry/api/wglext.h:78
WGL_NUMBER_OVERLAYS_ARB = 8200 	# http://www.opengl.org/registry/api/wglext.h:79
WGL_NUMBER_UNDERLAYS_ARB = 8201 	# http://www.opengl.org/registry/api/wglext.h:80
WGL_TRANSPARENT_ARB = 8202 	# http://www.opengl.org/registry/api/wglext.h:81
WGL_TRANSPARENT_RED_VALUE_ARB = 8247 	# http://www.opengl.org/registry/api/wglext.h:82
WGL_TRANSPARENT_GREEN_VALUE_ARB = 8248 	# http://www.opengl.org/registry/api/wglext.h:83
WGL_TRANSPARENT_BLUE_VALUE_ARB = 8249 	# http://www.opengl.org/registry/api/wglext.h:84
WGL_TRANSPARENT_ALPHA_VALUE_ARB = 8250 	# http://www.opengl.org/registry/api/wglext.h:85
WGL_TRANSPARENT_INDEX_VALUE_ARB = 8251 	# http://www.opengl.org/registry/api/wglext.h:86
WGL_SHARE_DEPTH_ARB = 8204 	# http://www.opengl.org/registry/api/wglext.h:87
WGL_SHARE_STENCIL_ARB = 8205 	# http://www.opengl.org/registry/api/wglext.h:88
WGL_SHARE_ACCUM_ARB = 8206 	# http://www.opengl.org/registry/api/wglext.h:89
WGL_SUPPORT_GDI_ARB = 8207 	# http://www.opengl.org/registry/api/wglext.h:90
WGL_SUPPORT_OPENGL_ARB = 8208 	# http://www.opengl.org/registry/api/wglext.h:91
WGL_DOUBLE_BUFFER_ARB = 8209 	# http://www.opengl.org/registry/api/wglext.h:92
WGL_STEREO_ARB = 8210 	# http://www.opengl.org/registry/api/wglext.h:93
WGL_PIXEL_TYPE_ARB = 8211 	# http://www.opengl.org/registry/api/wglext.h:94
WGL_COLOR_BITS_ARB = 8212 	# http://www.opengl.org/registry/api/wglext.h:95
WGL_RED_BITS_ARB = 8213 	# http://www.opengl.org/registry/api/wglext.h:96
WGL_RED_SHIFT_ARB = 8214 	# http://www.opengl.org/registry/api/wglext.h:97
WGL_GREEN_BITS_ARB = 8215 	# http://www.opengl.org/registry/api/wglext.h:98
WGL_GREEN_SHIFT_ARB = 8216 	# http://www.opengl.org/registry/api/wglext.h:99
WGL_BLUE_BITS_ARB = 8217 	# http://www.opengl.org/registry/api/wglext.h:100
WGL_BLUE_SHIFT_ARB = 8218 	# http://www.opengl.org/registry/api/wglext.h:101
WGL_ALPHA_BITS_ARB = 8219 	# http://www.opengl.org/registry/api/wglext.h:102
WGL_ALPHA_SHIFT_ARB = 8220 	# http://www.opengl.org/registry/api/wglext.h:103
WGL_ACCUM_BITS_ARB = 8221 	# http://www.opengl.org/registry/api/wglext.h:104
WGL_ACCUM_RED_BITS_ARB = 8222 	# http://www.opengl.org/registry/api/wglext.h:105
WGL_ACCUM_GREEN_BITS_ARB = 8223 	# http://www.opengl.org/registry/api/wglext.h:106
WGL_ACCUM_BLUE_BITS_ARB = 8224 	# http://www.opengl.org/registry/api/wglext.h:107
WGL_ACCUM_ALPHA_BITS_ARB = 8225 	# http://www.opengl.org/registry/api/wglext.h:108
WGL_DEPTH_BITS_ARB = 8226 	# http://www.opengl.org/registry/api/wglext.h:109
WGL_STENCIL_BITS_ARB = 8227 	# http://www.opengl.org/registry/api/wglext.h:110
WGL_AUX_BUFFERS_ARB = 8228 	# http://www.opengl.org/registry/api/wglext.h:111
WGL_NO_ACCELERATION_ARB = 8229 	# http://www.opengl.org/registry/api/wglext.h:112
WGL_GENERIC_ACCELERATION_ARB = 8230 	# http://www.opengl.org/registry/api/wglext.h:113
WGL_FULL_ACCELERATION_ARB = 8231 	# http://www.opengl.org/registry/api/wglext.h:114
WGL_SWAP_EXCHANGE_ARB = 8232 	# http://www.opengl.org/registry/api/wglext.h:115
WGL_SWAP_COPY_ARB = 8233 	# http://www.opengl.org/registry/api/wglext.h:116
WGL_SWAP_UNDEFINED_ARB = 8234 	# http://www.opengl.org/registry/api/wglext.h:117
WGL_TYPE_RGBA_ARB = 8235 	# http://www.opengl.org/registry/api/wglext.h:118
WGL_TYPE_COLORINDEX_ARB = 8236 	# http://www.opengl.org/registry/api/wglext.h:119
# ARB_make_current_read (http://www.opengl.org/registry/api/wglext.h:122)
ERROR_INVALID_PIXEL_TYPE_ARB = 8259 	# http://www.opengl.org/registry/api/wglext.h:123
ERROR_INCOMPATIBLE_DEVICE_CONTEXTS_ARB = 8276 	# http://www.opengl.org/registry/api/wglext.h:124
# ARB_pbuffer (http://www.opengl.org/registry/api/wglext.h:127)
WGL_DRAW_TO_PBUFFER_ARB = 8237 	# http://www.opengl.org/registry/api/wglext.h:128
WGL_MAX_PBUFFER_PIXELS_ARB = 8238 	# http://www.opengl.org/registry/api/wglext.h:129
WGL_MAX_PBUFFER_WIDTH_ARB = 8239 	# http://www.opengl.org/registry/api/wglext.h:130
WGL_MAX_PBUFFER_HEIGHT_ARB = 8240 	# http://www.opengl.org/registry/api/wglext.h:131
WGL_PBUFFER_LARGEST_ARB = 8243 	# http://www.opengl.org/registry/api/wglext.h:132
WGL_PBUFFER_WIDTH_ARB = 8244 	# http://www.opengl.org/registry/api/wglext.h:133
WGL_PBUFFER_HEIGHT_ARB = 8245 	# http://www.opengl.org/registry/api/wglext.h:134
WGL_PBUFFER_LOST_ARB = 8246 	# http://www.opengl.org/registry/api/wglext.h:135
# ARB_render_texture (http://www.opengl.org/registry/api/wglext.h:138)
WGL_BIND_TO_TEXTURE_RGB_ARB = 8304 	# http://www.opengl.org/registry/api/wglext.h:139
WGL_BIND_TO_TEXTURE_RGBA_ARB = 8305 	# http://www.opengl.org/registry/api/wglext.h:140
WGL_TEXTURE_FORMAT_ARB = 8306 	# http://www.opengl.org/registry/api/wglext.h:141
WGL_TEXTURE_TARGET_ARB = 8307 	# http://www.opengl.org/registry/api/wglext.h:142
WGL_MIPMAP_TEXTURE_ARB = 8308 	# http://www.opengl.org/registry/api/wglext.h:143
WGL_TEXTURE_RGB_ARB = 8309 	# http://www.opengl.org/registry/api/wglext.h:144
WGL_TEXTURE_RGBA_ARB = 8310 	# http://www.opengl.org/registry/api/wglext.h:145
WGL_NO_TEXTURE_ARB = 8311 	# http://www.opengl.org/registry/api/wglext.h:146
WGL_TEXTURE_CUBE_MAP_ARB = 8312 	# http://www.opengl.org/registry/api/wglext.h:147
WGL_TEXTURE_1D_ARB = 8313 	# http://www.opengl.org/registry/api/wglext.h:148
WGL_TEXTURE_2D_ARB = 8314 	# http://www.opengl.org/registry/api/wglext.h:149
WGL_MIPMAP_LEVEL_ARB = 8315 	# http://www.opengl.org/registry/api/wglext.h:150
WGL_CUBE_MAP_FACE_ARB = 8316 	# http://www.opengl.org/registry/api/wglext.h:151
WGL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB = 8317 	# http://www.opengl.org/registry/api/wglext.h:152
WGL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB = 8318 	# http://www.opengl.org/registry/api/wglext.h:153
WGL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB = 8319 	# http://www.opengl.org/registry/api/wglext.h:154
WGL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB = 8320 	# http://www.opengl.org/registry/api/wglext.h:155
WGL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB = 8321 	# http://www.opengl.org/registry/api/wglext.h:156
WGL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB = 8322 	# http://www.opengl.org/registry/api/wglext.h:157
WGL_FRONT_LEFT_ARB = 8323 	# http://www.opengl.org/registry/api/wglext.h:158
WGL_FRONT_RIGHT_ARB = 8324 	# http://www.opengl.org/registry/api/wglext.h:159
WGL_BACK_LEFT_ARB = 8325 	# http://www.opengl.org/registry/api/wglext.h:160
WGL_BACK_RIGHT_ARB = 8326 	# http://www.opengl.org/registry/api/wglext.h:161
WGL_AUX0_ARB = 8327 	# http://www.opengl.org/registry/api/wglext.h:162
WGL_AUX1_ARB = 8328 	# http://www.opengl.org/registry/api/wglext.h:163
WGL_AUX2_ARB = 8329 	# http://www.opengl.org/registry/api/wglext.h:164
WGL_AUX3_ARB = 8330 	# http://www.opengl.org/registry/api/wglext.h:165
WGL_AUX4_ARB = 8331 	# http://www.opengl.org/registry/api/wglext.h:166
WGL_AUX5_ARB = 8332 	# http://www.opengl.org/registry/api/wglext.h:167
WGL_AUX6_ARB = 8333 	# http://www.opengl.org/registry/api/wglext.h:168
WGL_AUX7_ARB = 8334 	# http://www.opengl.org/registry/api/wglext.h:169
WGL_AUX8_ARB = 8335 	# http://www.opengl.org/registry/api/wglext.h:170
WGL_AUX9_ARB = 8336 	# http://www.opengl.org/registry/api/wglext.h:171
# ARB_pixel_format_float (http://www.opengl.org/registry/api/wglext.h:174)
WGL_TYPE_RGBA_FLOAT_ARB = 8608 	# http://www.opengl.org/registry/api/wglext.h:175
# ARB_create_context (http://www.opengl.org/registry/api/wglext.h:178)
WGL_CONTEXT_DEBUG_BIT_ARB = 1 	# http://www.opengl.org/registry/api/wglext.h:179
WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB = 2 	# http://www.opengl.org/registry/api/wglext.h:180
WGL_CONTEXT_MAJOR_VERSION_ARB = 8337 	# http://www.opengl.org/registry/api/wglext.h:181
WGL_CONTEXT_MINOR_VERSION_ARB = 8338 	# http://www.opengl.org/registry/api/wglext.h:182
WGL_CONTEXT_LAYER_PLANE_ARB = 8339 	# http://www.opengl.org/registry/api/wglext.h:183
WGL_CONTEXT_FLAGS_ARB = 8340 	# http://www.opengl.org/registry/api/wglext.h:184
ERROR_INVALID_VERSION_ARB = 8341 	# http://www.opengl.org/registry/api/wglext.h:185
# EXT_make_current_read (http://www.opengl.org/registry/api/wglext.h:188)
ERROR_INVALID_PIXEL_TYPE_EXT = 8259 	# http://www.opengl.org/registry/api/wglext.h:189
# EXT_pixel_format (http://www.opengl.org/registry/api/wglext.h:192)
WGL_NUMBER_PIXEL_FORMATS_EXT = 8192 	# http://www.opengl.org/registry/api/wglext.h:193
WGL_DRAW_TO_WINDOW_EXT = 8193 	# http://www.opengl.org/registry/api/wglext.h:194
WGL_DRAW_TO_BITMAP_EXT = 8194 	# http://www.opengl.org/registry/api/wglext.h:195
WGL_ACCELERATION_EXT = 8195 	# http://www.opengl.org/registry/api/wglext.h:196
WGL_NEED_PALETTE_EXT = 8196 	# http://www.opengl.org/registry/api/wglext.h:197
WGL_NEED_SYSTEM_PALETTE_EXT = 8197 	# http://www.opengl.org/registry/api/wglext.h:198
WGL_SWAP_LAYER_BUFFERS_EXT = 8198 	# http://www.opengl.org/registry/api/wglext.h:199
WGL_SWAP_METHOD_EXT = 8199 	# http://www.opengl.org/registry/api/wglext.h:200
WGL_NUMBER_OVERLAYS_EXT = 8200 	# http://www.opengl.org/registry/api/wglext.h:201
WGL_NUMBER_UNDERLAYS_EXT = 8201 	# http://www.opengl.org/registry/api/wglext.h:202
WGL_TRANSPARENT_EXT = 8202 	# http://www.opengl.org/registry/api/wglext.h:203
WGL_TRANSPARENT_VALUE_EXT = 8203 	# http://www.opengl.org/registry/api/wglext.h:204
WGL_SHARE_DEPTH_EXT = 8204 	# http://www.opengl.org/registry/api/wglext.h:205
WGL_SHARE_STENCIL_EXT = 8205 	# http://www.opengl.org/registry/api/wglext.h:206
WGL_SHARE_ACCUM_EXT = 8206 	# http://www.opengl.org/registry/api/wglext.h:207
WGL_SUPPORT_GDI_EXT = 8207 	# http://www.opengl.org/registry/api/wglext.h:208
WGL_SUPPORT_OPENGL_EXT = 8208 	# http://www.opengl.org/registry/api/wglext.h:209
WGL_DOUBLE_BUFFER_EXT = 8209 	# http://www.opengl.org/registry/api/wglext.h:210
WGL_STEREO_EXT = 8210 	# http://www.opengl.org/registry/api/wglext.h:211
WGL_PIXEL_TYPE_EXT = 8211 	# http://www.opengl.org/registry/api/wglext.h:212
WGL_COLOR_BITS_EXT = 8212 	# http://www.opengl.org/registry/api/wglext.h:213
WGL_RED_BITS_EXT = 8213 	# http://www.opengl.org/registry/api/wglext.h:214
WGL_RED_SHIFT_EXT = 8214 	# http://www.opengl.org/registry/api/wglext.h:215
WGL_GREEN_BITS_EXT = 8215 	# http://www.opengl.org/registry/api/wglext.h:216
WGL_GREEN_SHIFT_EXT = 8216 	# http://www.opengl.org/registry/api/wglext.h:217
WGL_BLUE_BITS_EXT = 8217 	# http://www.opengl.org/registry/api/wglext.h:218
WGL_BLUE_SHIFT_EXT = 8218 	# http://www.opengl.org/registry/api/wglext.h:219
WGL_ALPHA_BITS_EXT = 8219 	# http://www.opengl.org/registry/api/wglext.h:220
WGL_ALPHA_SHIFT_EXT = 8220 	# http://www.opengl.org/registry/api/wglext.h:221
WGL_ACCUM_BITS_EXT = 8221 	# http://www.opengl.org/registry/api/wglext.h:222
WGL_ACCUM_RED_BITS_EXT = 8222 	# http://www.opengl.org/registry/api/wglext.h:223
WGL_ACCUM_GREEN_BITS_EXT = 8223 	# http://www.opengl.org/registry/api/wglext.h:224
WGL_ACCUM_BLUE_BITS_EXT = 8224 	# http://www.opengl.org/registry/api/wglext.h:225
WGL_ACCUM_ALPHA_BITS_EXT = 8225 	# http://www.opengl.org/registry/api/wglext.h:226
WGL_DEPTH_BITS_EXT = 8226 	# http://www.opengl.org/registry/api/wglext.h:227
WGL_STENCIL_BITS_EXT = 8227 	# http://www.opengl.org/registry/api/wglext.h:228
WGL_AUX_BUFFERS_EXT = 8228 	# http://www.opengl.org/registry/api/wglext.h:229
WGL_NO_ACCELERATION_EXT = 8229 	# http://www.opengl.org/registry/api/wglext.h:230
WGL_GENERIC_ACCELERATION_EXT = 8230 	# http://www.opengl.org/registry/api/wglext.h:231
WGL_FULL_ACCELERATION_EXT = 8231 	# http://www.opengl.org/registry/api/wglext.h:232
WGL_SWAP_EXCHANGE_EXT = 8232 	# http://www.opengl.org/registry/api/wglext.h:233
WGL_SWAP_COPY_EXT = 8233 	# http://www.opengl.org/registry/api/wglext.h:234
WGL_SWAP_UNDEFINED_EXT = 8234 	# http://www.opengl.org/registry/api/wglext.h:235
WGL_TYPE_RGBA_EXT = 8235 	# http://www.opengl.org/registry/api/wglext.h:236
WGL_TYPE_COLORINDEX_EXT = 8236 	# http://www.opengl.org/registry/api/wglext.h:237
# EXT_pbuffer (http://www.opengl.org/registry/api/wglext.h:240)
WGL_DRAW_TO_PBUFFER_EXT = 8237 	# http://www.opengl.org/registry/api/wglext.h:241
WGL_MAX_PBUFFER_PIXELS_EXT = 8238 	# http://www.opengl.org/registry/api/wglext.h:242
WGL_MAX_PBUFFER_WIDTH_EXT = 8239 	# http://www.opengl.org/registry/api/wglext.h:243
WGL_MAX_PBUFFER_HEIGHT_EXT = 8240 	# http://www.opengl.org/registry/api/wglext.h:244
WGL_OPTIMAL_PBUFFER_WIDTH_EXT = 8241 	# http://www.opengl.org/registry/api/wglext.h:245
WGL_OPTIMAL_PBUFFER_HEIGHT_EXT = 8242 	# http://www.opengl.org/registry/api/wglext.h:246
WGL_PBUFFER_LARGEST_EXT = 8243 	# http://www.opengl.org/registry/api/wglext.h:247
WGL_PBUFFER_WIDTH_EXT = 8244 	# http://www.opengl.org/registry/api/wglext.h:248
WGL_PBUFFER_HEIGHT_EXT = 8245 	# http://www.opengl.org/registry/api/wglext.h:249
# EXT_depth_float (http://www.opengl.org/registry/api/wglext.h:252)
WGL_DEPTH_FLOAT_EXT = 8256 	# http://www.opengl.org/registry/api/wglext.h:253
# 3DFX_multisample (http://www.opengl.org/registry/api/wglext.h:256)
WGL_SAMPLE_BUFFERS_3DFX = 8288 	# http://www.opengl.org/registry/api/wglext.h:257
WGL_SAMPLES_3DFX = 8289 	# http://www.opengl.org/registry/api/wglext.h:258
# EXT_multisample (http://www.opengl.org/registry/api/wglext.h:261)
WGL_SAMPLE_BUFFERS_EXT = 8257 	# http://www.opengl.org/registry/api/wglext.h:262
WGL_SAMPLES_EXT = 8258 	# http://www.opengl.org/registry/api/wglext.h:263
# I3D_digital_video_control (http://www.opengl.org/registry/api/wglext.h:266)
WGL_DIGITAL_VIDEO_CURSOR_ALPHA_FRAMEBUFFER_I3D = 8272 	# http://www.opengl.org/registry/api/wglext.h:267
WGL_DIGITAL_VIDEO_CURSOR_ALPHA_VALUE_I3D = 8273 	# http://www.opengl.org/registry/api/wglext.h:268
WGL_DIGITAL_VIDEO_CURSOR_INCLUDED_I3D = 8274 	# http://www.opengl.org/registry/api/wglext.h:269
WGL_DIGITAL_VIDEO_GAMMA_CORRECTED_I3D = 8275 	# http://www.opengl.org/registry/api/wglext.h:270
# I3D_gamma (http://www.opengl.org/registry/api/wglext.h:273)
WGL_GAMMA_TABLE_SIZE_I3D = 8270 	# http://www.opengl.org/registry/api/wglext.h:274
WGL_GAMMA_EXCLUDE_DESKTOP_I3D = 8271 	# http://www.opengl.org/registry/api/wglext.h:275
# I3D_genlock (http://www.opengl.org/registry/api/wglext.h:278)
WGL_GENLOCK_SOURCE_MULTIVIEW_I3D = 8260 	# http://www.opengl.org/registry/api/wglext.h:279
WGL_GENLOCK_SOURCE_EXTENAL_SYNC_I3D = 8261 	# http://www.opengl.org/registry/api/wglext.h:280
WGL_GENLOCK_SOURCE_EXTENAL_FIELD_I3D = 8262 	# http://www.opengl.org/registry/api/wglext.h:281
WGL_GENLOCK_SOURCE_EXTENAL_TTL_I3D = 8263 	# http://www.opengl.org/registry/api/wglext.h:282
WGL_GENLOCK_SOURCE_DIGITAL_SYNC_I3D = 8264 	# http://www.opengl.org/registry/api/wglext.h:283
WGL_GENLOCK_SOURCE_DIGITAL_FIELD_I3D = 8265 	# http://www.opengl.org/registry/api/wglext.h:284
WGL_GENLOCK_SOURCE_EDGE_FALLING_I3D = 8266 	# http://www.opengl.org/registry/api/wglext.h:285
WGL_GENLOCK_SOURCE_EDGE_RISING_I3D = 8267 	# http://www.opengl.org/registry/api/wglext.h:286
WGL_GENLOCK_SOURCE_EDGE_BOTH_I3D = 8268 	# http://www.opengl.org/registry/api/wglext.h:287
# I3D_image_buffer (http://www.opengl.org/registry/api/wglext.h:290)
WGL_IMAGE_BUFFER_MIN_ACCESS_I3D = 1 	# http://www.opengl.org/registry/api/wglext.h:291
WGL_IMAGE_BUFFER_LOCK_I3D = 2 	# http://www.opengl.org/registry/api/wglext.h:292
# I3D_swap_frame_lock (http://www.opengl.org/registry/api/wglext.h:295)
# NV_render_depth_texture (http://www.opengl.org/registry/api/wglext.h:298)
WGL_BIND_TO_TEXTURE_DEPTH_NV = 8355 	# http://www.opengl.org/registry/api/wglext.h:299
WGL_BIND_TO_TEXTURE_RECTANGLE_DEPTH_NV = 8356 	# http://www.opengl.org/registry/api/wglext.h:300
WGL_DEPTH_TEXTURE_FORMAT_NV = 8357 	# http://www.opengl.org/registry/api/wglext.h:301
WGL_TEXTURE_DEPTH_COMPONENT_NV = 8358 	# http://www.opengl.org/registry/api/wglext.h:302
WGL_DEPTH_COMPONENT_NV = 8359 	# http://www.opengl.org/registry/api/wglext.h:303
# NV_render_texture_rectangle (http://www.opengl.org/registry/api/wglext.h:306)
WGL_BIND_TO_TEXTURE_RECTANGLE_RGB_NV = 8352 	# http://www.opengl.org/registry/api/wglext.h:307
WGL_BIND_TO_TEXTURE_RECTANGLE_RGBA_NV = 8353 	# http://www.opengl.org/registry/api/wglext.h:308
WGL_TEXTURE_RECTANGLE_NV = 8354 	# http://www.opengl.org/registry/api/wglext.h:309
# ATI_pixel_format_float (http://www.opengl.org/registry/api/wglext.h:312)
WGL_TYPE_RGBA_FLOAT_ATI = 8608 	# http://www.opengl.org/registry/api/wglext.h:313
# NV_float_buffer (http://www.opengl.org/registry/api/wglext.h:316)
WGL_FLOAT_COMPONENTS_NV = 8368 	# http://www.opengl.org/registry/api/wglext.h:317
WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_R_NV = 8369 	# http://www.opengl.org/registry/api/wglext.h:318
WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RG_NV = 8370 	# http://www.opengl.org/registry/api/wglext.h:319
WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RGB_NV = 8371 	# http://www.opengl.org/registry/api/wglext.h:320
WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RGBA_NV = 8372 	# http://www.opengl.org/registry/api/wglext.h:321
WGL_TEXTURE_FLOAT_R_NV = 8373 	# http://www.opengl.org/registry/api/wglext.h:322
WGL_TEXTURE_FLOAT_RG_NV = 8374 	# http://www.opengl.org/registry/api/wglext.h:323
WGL_TEXTURE_FLOAT_RGB_NV = 8375 	# http://www.opengl.org/registry/api/wglext.h:324
WGL_TEXTURE_FLOAT_RGBA_NV = 8376 	# http://www.opengl.org/registry/api/wglext.h:325
# 3DL_stereo_control (http://www.opengl.org/registry/api/wglext.h:328)
WGL_STEREO_EMITTER_ENABLE_3DL = 8277 	# http://www.opengl.org/registry/api/wglext.h:329
WGL_STEREO_EMITTER_DISABLE_3DL = 8278 	# http://www.opengl.org/registry/api/wglext.h:330
WGL_STEREO_POLARITY_NORMAL_3DL = 8279 	# http://www.opengl.org/registry/api/wglext.h:331
WGL_STEREO_POLARITY_INVERT_3DL = 8280 	# http://www.opengl.org/registry/api/wglext.h:332
# EXT_pixel_format_packed_float (http://www.opengl.org/registry/api/wglext.h:335)
WGL_TYPE_RGBA_UNSIGNED_FLOAT_EXT = 8360 	# http://www.opengl.org/registry/api/wglext.h:336
# EXT_framebuffer_sRGB (http://www.opengl.org/registry/api/wglext.h:339)
WGL_FRAMEBUFFER_SRGB_CAPABLE_EXT = 8361 	# http://www.opengl.org/registry/api/wglext.h:340
# NV_present_video (http://www.opengl.org/registry/api/wglext.h:343)
WGL_NUM_VIDEO_SLOTS_NV = 8432 	# http://www.opengl.org/registry/api/wglext.h:344
# NV_video_out (http://www.opengl.org/registry/api/wglext.h:347)
WGL_BIND_TO_VIDEO_RGB_NV = 8384 	# http://www.opengl.org/registry/api/wglext.h:348
WGL_BIND_TO_VIDEO_RGBA_NV = 8385 	# http://www.opengl.org/registry/api/wglext.h:349
WGL_BIND_TO_VIDEO_RGB_AND_DEPTH_NV = 8386 	# http://www.opengl.org/registry/api/wglext.h:350
WGL_VIDEO_OUT_COLOR_NV = 8387 	# http://www.opengl.org/registry/api/wglext.h:351
WGL_VIDEO_OUT_ALPHA_NV = 8388 	# http://www.opengl.org/registry/api/wglext.h:352
WGL_VIDEO_OUT_DEPTH_NV = 8389 	# http://www.opengl.org/registry/api/wglext.h:353
WGL_VIDEO_OUT_COLOR_AND_ALPHA_NV = 8390 	# http://www.opengl.org/registry/api/wglext.h:354
WGL_VIDEO_OUT_COLOR_AND_DEPTH_NV = 8391 	# http://www.opengl.org/registry/api/wglext.h:355
WGL_VIDEO_OUT_FRAME = 8392 	# http://www.opengl.org/registry/api/wglext.h:356
WGL_VIDEO_OUT_FIELD_1 = 8393 	# http://www.opengl.org/registry/api/wglext.h:357
WGL_VIDEO_OUT_FIELD_2 = 8394 	# http://www.opengl.org/registry/api/wglext.h:358
WGL_VIDEO_OUT_STACKED_FIELDS_1_2 = 8395 	# http://www.opengl.org/registry/api/wglext.h:359
WGL_VIDEO_OUT_STACKED_FIELDS_2_1 = 8396 	# http://www.opengl.org/registry/api/wglext.h:360
# NV_swap_group (http://www.opengl.org/registry/api/wglext.h:363)
# NV_gpu_affinity (http://www.opengl.org/registry/api/wglext.h:366)
WGL_ERROR_INCOMPATIBLE_AFFINITY_MASKS_NV = 8400 	# http://www.opengl.org/registry/api/wglext.h:367
WGL_ERROR_MISSING_AFFINITY_MASK_NV = 8401 	# http://www.opengl.org/registry/api/wglext.h:368
# ARB_pbuffer (http://www.opengl.org/registry/api/wglext.h:374)

HPBUFFERARB = HANDLE 	# http://www.opengl.org/registry/api/wglext.h:375
# EXT_pbuffer (http://www.opengl.org/registry/api/wglext.h:377)
HPBUFFEREXT = HANDLE 	# http://www.opengl.org/registry/api/wglext.h:378
# NV_present_video (http://www.opengl.org/registry/api/wglext.h:380)
HVIDEOOUTPUTDEVICENV = HANDLE 	# http://www.opengl.org/registry/api/wglext.h:381
# NV_video_out (http://www.opengl.org/registry/api/wglext.h:383)
HPVIDEODEV = HANDLE 	# http://www.opengl.org/registry/api/wglext.h:384
# NV_gpu_affinity (http://www.opengl.org/registry/api/wglext.h:386)
HPGPUNV = HANDLE 	# http://www.opengl.org/registry/api/wglext.h:387
HGPUNV = HANDLE 	# http://www.opengl.org/registry/api/wglext.h:388
class struct__GPU_DEVICE(Structure):
    __slots__ = [
        'DeviceName',
        'DeviceString',
        'Flags',
        'cb',
        'rcVirtualScreen',
    ]

class struct_tagRECT(Structure):
    __slots__ = [
        'bottom',
        'left',
        'right',
        'top',
    ]

struct_tagRECT._fields_ = [
    ('left', LONG),
    ('top', LONG),
    ('right', LONG),
    ('bottom', LONG),
]

RECT = struct_tagRECT 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:200
struct__GPU_DEVICE._fields_ = [
    ('cb', DWORD),
    ('DeviceName', CHAR * 32),
    ('DeviceString', CHAR * 128),
    ('Flags', DWORD),
    ('rcVirtualScreen', RECT),
]

GPU_DEVICE = struct__GPU_DEVICE 	# http://www.opengl.org/registry/api/wglext.h:396
PGPU_DEVICE = POINTER(struct__GPU_DEVICE) 	# http://www.opengl.org/registry/api/wglext.h:396
# ARB_buffer_region (http://www.opengl.org/registry/api/wglext.h:399)
WGL_ARB_buffer_region = 1 	# http://www.opengl.org/registry/api/wglext.h:400

# http://www.opengl.org/registry/api/wglext.h:402
wglCreateBufferRegionARB = _link_function('wglCreateBufferRegionARB', HANDLE, [HDC, c_int, UINT], 'ARB_buffer_region')

VOID = None 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:45
# http://www.opengl.org/registry/api/wglext.h:403
wglDeleteBufferRegionARB = _link_function('wglDeleteBufferRegionARB', VOID, [HANDLE], 'ARB_buffer_region')

BOOL = c_long 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:52
# http://www.opengl.org/registry/api/wglext.h:404
wglSaveBufferRegionARB = _link_function('wglSaveBufferRegionARB', BOOL, [HANDLE, c_int, c_int, c_int, c_int], 'ARB_buffer_region')

# http://www.opengl.org/registry/api/wglext.h:405
wglRestoreBufferRegionARB = _link_function('wglRestoreBufferRegionARB', BOOL, [HANDLE, c_int, c_int, c_int, c_int, c_int, c_int], 'ARB_buffer_region')

PFNWGLCREATEBUFFERREGIONARBPROC = CFUNCTYPE(HANDLE, HDC, c_int, UINT) 	# http://www.opengl.org/registry/api/wglext.h:407
PFNWGLDELETEBUFFERREGIONARBPROC = CFUNCTYPE(VOID, HANDLE) 	# http://www.opengl.org/registry/api/wglext.h:408
PFNWGLSAVEBUFFERREGIONARBPROC = CFUNCTYPE(BOOL, HANDLE, c_int, c_int, c_int, c_int) 	# http://www.opengl.org/registry/api/wglext.h:409
PFNWGLRESTOREBUFFERREGIONARBPROC = CFUNCTYPE(BOOL, HANDLE, c_int, c_int, c_int, c_int, c_int, c_int) 	# http://www.opengl.org/registry/api/wglext.h:410
# ARB_multisample (http://www.opengl.org/registry/api/wglext.h:413)
WGL_ARB_multisample = 1 	# http://www.opengl.org/registry/api/wglext.h:414
# ARB_extensions_string (http://www.opengl.org/registry/api/wglext.h:417)
WGL_ARB_extensions_string = 1 	# http://www.opengl.org/registry/api/wglext.h:418
# http://www.opengl.org/registry/api/wglext.h:420
wglGetExtensionsStringARB = _link_function('wglGetExtensionsStringARB', c_char_p, [HDC], 'ARB_extensions_string')

PFNWGLGETEXTENSIONSSTRINGARBPROC = CFUNCTYPE(c_char_p, HDC) 	# http://www.opengl.org/registry/api/wglext.h:422
# ARB_pixel_format (http://www.opengl.org/registry/api/wglext.h:425)
WGL_ARB_pixel_format = 1 	# http://www.opengl.org/registry/api/wglext.h:426
# http://www.opengl.org/registry/api/wglext.h:428
wglGetPixelFormatAttribivARB = _link_function('wglGetPixelFormatAttribivARB', BOOL, [HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(c_int)], 'ARB_pixel_format')

# http://www.opengl.org/registry/api/wglext.h:429
wglGetPixelFormatAttribfvARB = _link_function('wglGetPixelFormatAttribfvARB', BOOL, [HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(FLOAT)], 'ARB_pixel_format')

# http://www.opengl.org/registry/api/wglext.h:430
wglChoosePixelFormatARB = _link_function('wglChoosePixelFormatARB', BOOL, [HDC, POINTER(c_int), POINTER(FLOAT), UINT, POINTER(c_int), POINTER(UINT)], 'ARB_pixel_format')

PFNWGLGETPIXELFORMATATTRIBIVARBPROC = CFUNCTYPE(BOOL, HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:432
PFNWGLGETPIXELFORMATATTRIBFVARBPROC = CFUNCTYPE(BOOL, HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(FLOAT)) 	# http://www.opengl.org/registry/api/wglext.h:433
PFNWGLCHOOSEPIXELFORMATARBPROC = CFUNCTYPE(BOOL, HDC, POINTER(c_int), POINTER(FLOAT), UINT, POINTER(c_int), POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:434
# ARB_make_current_read (http://www.opengl.org/registry/api/wglext.h:437)
WGL_ARB_make_current_read = 1 	# http://www.opengl.org/registry/api/wglext.h:438
HGLRC = HANDLE 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:60
# http://www.opengl.org/registry/api/wglext.h:440
wglMakeContextCurrentARB = _link_function('wglMakeContextCurrentARB', BOOL, [HDC, HDC, HGLRC], 'ARB_make_current_read')

# http://www.opengl.org/registry/api/wglext.h:441
wglGetCurrentReadDCARB = _link_function('wglGetCurrentReadDCARB', HDC, [], 'ARB_make_current_read')

PFNWGLMAKECONTEXTCURRENTARBPROC = CFUNCTYPE(BOOL, HDC, HDC, HGLRC) 	# http://www.opengl.org/registry/api/wglext.h:443
PFNWGLGETCURRENTREADDCARBPROC = CFUNCTYPE(HDC) 	# http://www.opengl.org/registry/api/wglext.h:444
# ARB_pbuffer (http://www.opengl.org/registry/api/wglext.h:447)
WGL_ARB_pbuffer = 1 	# http://www.opengl.org/registry/api/wglext.h:448
# http://www.opengl.org/registry/api/wglext.h:450
wglCreatePbufferARB = _link_function('wglCreatePbufferARB', HPBUFFERARB, [HDC, c_int, c_int, c_int, POINTER(c_int)], 'ARB_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:451
wglGetPbufferDCARB = _link_function('wglGetPbufferDCARB', HDC, [HPBUFFERARB], 'ARB_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:452
wglReleasePbufferDCARB = _link_function('wglReleasePbufferDCARB', c_int, [HPBUFFERARB, HDC], 'ARB_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:453
wglDestroyPbufferARB = _link_function('wglDestroyPbufferARB', BOOL, [HPBUFFERARB], 'ARB_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:454
wglQueryPbufferARB = _link_function('wglQueryPbufferARB', BOOL, [HPBUFFERARB, c_int, POINTER(c_int)], 'ARB_pbuffer')

PFNWGLCREATEPBUFFERARBPROC = CFUNCTYPE(HPBUFFERARB, HDC, c_int, c_int, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:456
PFNWGLGETPBUFFERDCARBPROC = CFUNCTYPE(HDC, HPBUFFERARB) 	# http://www.opengl.org/registry/api/wglext.h:457
PFNWGLRELEASEPBUFFERDCARBPROC = CFUNCTYPE(c_int, HPBUFFERARB, HDC) 	# http://www.opengl.org/registry/api/wglext.h:458
PFNWGLDESTROYPBUFFERARBPROC = CFUNCTYPE(BOOL, HPBUFFERARB) 	# http://www.opengl.org/registry/api/wglext.h:459
PFNWGLQUERYPBUFFERARBPROC = CFUNCTYPE(BOOL, HPBUFFERARB, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:460
# ARB_render_texture (http://www.opengl.org/registry/api/wglext.h:463)
WGL_ARB_render_texture = 1 	# http://www.opengl.org/registry/api/wglext.h:464
# http://www.opengl.org/registry/api/wglext.h:466
wglBindTexImageARB = _link_function('wglBindTexImageARB', BOOL, [HPBUFFERARB, c_int], 'ARB_render_texture')

# http://www.opengl.org/registry/api/wglext.h:467
wglReleaseTexImageARB = _link_function('wglReleaseTexImageARB', BOOL, [HPBUFFERARB, c_int], 'ARB_render_texture')

# http://www.opengl.org/registry/api/wglext.h:468
wglSetPbufferAttribARB = _link_function('wglSetPbufferAttribARB', BOOL, [HPBUFFERARB, POINTER(c_int)], 'ARB_render_texture')

PFNWGLBINDTEXIMAGEARBPROC = CFUNCTYPE(BOOL, HPBUFFERARB, c_int) 	# http://www.opengl.org/registry/api/wglext.h:470
PFNWGLRELEASETEXIMAGEARBPROC = CFUNCTYPE(BOOL, HPBUFFERARB, c_int) 	# http://www.opengl.org/registry/api/wglext.h:471
PFNWGLSETPBUFFERATTRIBARBPROC = CFUNCTYPE(BOOL, HPBUFFERARB, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:472
# ARB_pixel_format_float (http://www.opengl.org/registry/api/wglext.h:475)
WGL_ARB_pixel_format_float = 1 	# http://www.opengl.org/registry/api/wglext.h:476
# ARB_create_context (http://www.opengl.org/registry/api/wglext.h:479)
WGL_ARB_create_context = 1 	# http://www.opengl.org/registry/api/wglext.h:480
# http://www.opengl.org/registry/api/wglext.h:482
wglCreateContextAttribsARB = _link_function('wglCreateContextAttribsARB', HGLRC, [HDC, HGLRC, POINTER(c_int)], 'ARB_create_context')

PFNWGLCREATECONTEXTATTRIBSARBPROC = CFUNCTYPE(HGLRC, HDC, HGLRC, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:484
# EXT_display_color_table (http://www.opengl.org/registry/api/wglext.h:487)
WGL_EXT_display_color_table = 1 	# http://www.opengl.org/registry/api/wglext.h:488
GLboolean = c_ubyte 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:18
GLushort = c_ushort 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:25
# http://www.opengl.org/registry/api/wglext.h:490
wglCreateDisplayColorTableEXT = _link_function('wglCreateDisplayColorTableEXT', GLboolean, [GLushort], 'EXT_display_color_table')

GLuint = c_uint 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:26
# http://www.opengl.org/registry/api/wglext.h:491
wglLoadDisplayColorTableEXT = _link_function('wglLoadDisplayColorTableEXT', GLboolean, [POINTER(GLushort), GLuint], 'EXT_display_color_table')

# http://www.opengl.org/registry/api/wglext.h:492
wglBindDisplayColorTableEXT = _link_function('wglBindDisplayColorTableEXT', GLboolean, [GLushort], 'EXT_display_color_table')

# http://www.opengl.org/registry/api/wglext.h:493
wglDestroyDisplayColorTableEXT = _link_function('wglDestroyDisplayColorTableEXT', VOID, [GLushort], 'EXT_display_color_table')

PFNWGLCREATEDISPLAYCOLORTABLEEXTPROC = CFUNCTYPE(GLboolean, GLushort) 	# http://www.opengl.org/registry/api/wglext.h:495
PFNWGLLOADDISPLAYCOLORTABLEEXTPROC = CFUNCTYPE(GLboolean, POINTER(GLushort), GLuint) 	# http://www.opengl.org/registry/api/wglext.h:496
PFNWGLBINDDISPLAYCOLORTABLEEXTPROC = CFUNCTYPE(GLboolean, GLushort) 	# http://www.opengl.org/registry/api/wglext.h:497
PFNWGLDESTROYDISPLAYCOLORTABLEEXTPROC = CFUNCTYPE(VOID, GLushort) 	# http://www.opengl.org/registry/api/wglext.h:498
# EXT_extensions_string (http://www.opengl.org/registry/api/wglext.h:501)
WGL_EXT_extensions_string = 1 	# http://www.opengl.org/registry/api/wglext.h:502
# http://www.opengl.org/registry/api/wglext.h:504
wglGetExtensionsStringEXT = _link_function('wglGetExtensionsStringEXT', c_char_p, [], 'EXT_extensions_string')

PFNWGLGETEXTENSIONSSTRINGEXTPROC = CFUNCTYPE(c_char_p) 	# http://www.opengl.org/registry/api/wglext.h:506
# EXT_make_current_read (http://www.opengl.org/registry/api/wglext.h:509)
WGL_EXT_make_current_read = 1 	# http://www.opengl.org/registry/api/wglext.h:510
# http://www.opengl.org/registry/api/wglext.h:512
wglMakeContextCurrentEXT = _link_function('wglMakeContextCurrentEXT', BOOL, [HDC, HDC, HGLRC], 'EXT_make_current_read')

# http://www.opengl.org/registry/api/wglext.h:513
wglGetCurrentReadDCEXT = _link_function('wglGetCurrentReadDCEXT', HDC, [], 'EXT_make_current_read')

PFNWGLMAKECONTEXTCURRENTEXTPROC = CFUNCTYPE(BOOL, HDC, HDC, HGLRC) 	# http://www.opengl.org/registry/api/wglext.h:515
PFNWGLGETCURRENTREADDCEXTPROC = CFUNCTYPE(HDC) 	# http://www.opengl.org/registry/api/wglext.h:516
# EXT_pbuffer (http://www.opengl.org/registry/api/wglext.h:519)
WGL_EXT_pbuffer = 1 	# http://www.opengl.org/registry/api/wglext.h:520
# http://www.opengl.org/registry/api/wglext.h:522
wglCreatePbufferEXT = _link_function('wglCreatePbufferEXT', HPBUFFEREXT, [HDC, c_int, c_int, c_int, POINTER(c_int)], 'EXT_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:523
wglGetPbufferDCEXT = _link_function('wglGetPbufferDCEXT', HDC, [HPBUFFEREXT], 'EXT_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:524
wglReleasePbufferDCEXT = _link_function('wglReleasePbufferDCEXT', c_int, [HPBUFFEREXT, HDC], 'EXT_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:525
wglDestroyPbufferEXT = _link_function('wglDestroyPbufferEXT', BOOL, [HPBUFFEREXT], 'EXT_pbuffer')

# http://www.opengl.org/registry/api/wglext.h:526
wglQueryPbufferEXT = _link_function('wglQueryPbufferEXT', BOOL, [HPBUFFEREXT, c_int, POINTER(c_int)], 'EXT_pbuffer')

PFNWGLCREATEPBUFFEREXTPROC = CFUNCTYPE(HPBUFFEREXT, HDC, c_int, c_int, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:528
PFNWGLGETPBUFFERDCEXTPROC = CFUNCTYPE(HDC, HPBUFFEREXT) 	# http://www.opengl.org/registry/api/wglext.h:529
PFNWGLRELEASEPBUFFERDCEXTPROC = CFUNCTYPE(c_int, HPBUFFEREXT, HDC) 	# http://www.opengl.org/registry/api/wglext.h:530
PFNWGLDESTROYPBUFFEREXTPROC = CFUNCTYPE(BOOL, HPBUFFEREXT) 	# http://www.opengl.org/registry/api/wglext.h:531
PFNWGLQUERYPBUFFEREXTPROC = CFUNCTYPE(BOOL, HPBUFFEREXT, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:532
# EXT_pixel_format (http://www.opengl.org/registry/api/wglext.h:535)
WGL_EXT_pixel_format = 1 	# http://www.opengl.org/registry/api/wglext.h:536
# http://www.opengl.org/registry/api/wglext.h:538
wglGetPixelFormatAttribivEXT = _link_function('wglGetPixelFormatAttribivEXT', BOOL, [HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(c_int)], 'EXT_pixel_format')

# http://www.opengl.org/registry/api/wglext.h:539
wglGetPixelFormatAttribfvEXT = _link_function('wglGetPixelFormatAttribfvEXT', BOOL, [HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(FLOAT)], 'EXT_pixel_format')

# http://www.opengl.org/registry/api/wglext.h:540
wglChoosePixelFormatEXT = _link_function('wglChoosePixelFormatEXT', BOOL, [HDC, POINTER(c_int), POINTER(FLOAT), UINT, POINTER(c_int), POINTER(UINT)], 'EXT_pixel_format')

PFNWGLGETPIXELFORMATATTRIBIVEXTPROC = CFUNCTYPE(BOOL, HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:542
PFNWGLGETPIXELFORMATATTRIBFVEXTPROC = CFUNCTYPE(BOOL, HDC, c_int, c_int, UINT, POINTER(c_int), POINTER(FLOAT)) 	# http://www.opengl.org/registry/api/wglext.h:543
PFNWGLCHOOSEPIXELFORMATEXTPROC = CFUNCTYPE(BOOL, HDC, POINTER(c_int), POINTER(FLOAT), UINT, POINTER(c_int), POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:544
# EXT_swap_control (http://www.opengl.org/registry/api/wglext.h:547)
WGL_EXT_swap_control = 1 	# http://www.opengl.org/registry/api/wglext.h:548
# http://www.opengl.org/registry/api/wglext.h:550
wglSwapIntervalEXT = _link_function('wglSwapIntervalEXT', BOOL, [c_int], 'EXT_swap_control')

# http://www.opengl.org/registry/api/wglext.h:551
wglGetSwapIntervalEXT = _link_function('wglGetSwapIntervalEXT', c_int, [], 'EXT_swap_control')

PFNWGLSWAPINTERVALEXTPROC = CFUNCTYPE(BOOL, c_int) 	# http://www.opengl.org/registry/api/wglext.h:553
PFNWGLGETSWAPINTERVALEXTPROC = CFUNCTYPE(c_int) 	# http://www.opengl.org/registry/api/wglext.h:554
# EXT_depth_float (http://www.opengl.org/registry/api/wglext.h:557)
WGL_EXT_depth_float = 1 	# http://www.opengl.org/registry/api/wglext.h:558
# NV_vertex_array_range (http://www.opengl.org/registry/api/wglext.h:561)
WGL_NV_vertex_array_range = 1 	# http://www.opengl.org/registry/api/wglext.h:562
GLsizei = c_int 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:23
GLfloat = c_float 	# C:\cygwin\home\alex\projects\pyglet\tools\wgl.h:27
# http://www.opengl.org/registry/api/wglext.h:564
wglAllocateMemoryNV = _link_function('wglAllocateMemoryNV', POINTER(c_void), [GLsizei, GLfloat, GLfloat, GLfloat], 'NV_vertex_array_range')

# http://www.opengl.org/registry/api/wglext.h:565
wglFreeMemoryNV = _link_function('wglFreeMemoryNV', None, [POINTER(None)], 'NV_vertex_array_range')

PFNWGLALLOCATEMEMORYNVPROC = CFUNCTYPE(POINTER(c_void), GLsizei, GLfloat, GLfloat, GLfloat) 	# http://www.opengl.org/registry/api/wglext.h:567
PFNWGLFREEMEMORYNVPROC = CFUNCTYPE(None, POINTER(None)) 	# http://www.opengl.org/registry/api/wglext.h:568
# 3DFX_multisample (http://www.opengl.org/registry/api/wglext.h:571)
WGL_3DFX_multisample = 1 	# http://www.opengl.org/registry/api/wglext.h:572
# EXT_multisample (http://www.opengl.org/registry/api/wglext.h:575)
WGL_EXT_multisample = 1 	# http://www.opengl.org/registry/api/wglext.h:576
# OML_sync_control (http://www.opengl.org/registry/api/wglext.h:579)
WGL_OML_sync_control = 1 	# http://www.opengl.org/registry/api/wglext.h:580

# http://www.opengl.org/registry/api/wglext.h:582
wglGetSyncValuesOML = _link_function('wglGetSyncValuesOML', BOOL, [HDC, POINTER(INT64), POINTER(INT64), POINTER(INT64)], 'OML_sync_control')

# http://www.opengl.org/registry/api/wglext.h:583
wglGetMscRateOML = _link_function('wglGetMscRateOML', BOOL, [HDC, POINTER(INT32), POINTER(INT32)], 'OML_sync_control')

# http://www.opengl.org/registry/api/wglext.h:584
wglSwapBuffersMscOML = _link_function('wglSwapBuffersMscOML', INT64, [HDC, INT64, INT64, INT64], 'OML_sync_control')

# http://www.opengl.org/registry/api/wglext.h:585
wglSwapLayerBuffersMscOML = _link_function('wglSwapLayerBuffersMscOML', INT64, [HDC, c_int, INT64, INT64, INT64], 'OML_sync_control')

# http://www.opengl.org/registry/api/wglext.h:586
wglWaitForMscOML = _link_function('wglWaitForMscOML', BOOL, [HDC, INT64, INT64, INT64, POINTER(INT64), POINTER(INT64), POINTER(INT64)], 'OML_sync_control')

# http://www.opengl.org/registry/api/wglext.h:587
wglWaitForSbcOML = _link_function('wglWaitForSbcOML', BOOL, [HDC, INT64, POINTER(INT64), POINTER(INT64), POINTER(INT64)], 'OML_sync_control')

PFNWGLGETSYNCVALUESOMLPROC = CFUNCTYPE(BOOL, HDC, POINTER(INT64), POINTER(INT64), POINTER(INT64)) 	# http://www.opengl.org/registry/api/wglext.h:589
PFNWGLGETMSCRATEOMLPROC = CFUNCTYPE(BOOL, HDC, POINTER(INT32), POINTER(INT32)) 	# http://www.opengl.org/registry/api/wglext.h:590
PFNWGLSWAPBUFFERSMSCOMLPROC = CFUNCTYPE(INT64, HDC, INT64, INT64, INT64) 	# http://www.opengl.org/registry/api/wglext.h:591
PFNWGLSWAPLAYERBUFFERSMSCOMLPROC = CFUNCTYPE(INT64, HDC, c_int, INT64, INT64, INT64) 	# http://www.opengl.org/registry/api/wglext.h:592
PFNWGLWAITFORMSCOMLPROC = CFUNCTYPE(BOOL, HDC, INT64, INT64, INT64, POINTER(INT64), POINTER(INT64), POINTER(INT64)) 	# http://www.opengl.org/registry/api/wglext.h:593
PFNWGLWAITFORSBCOMLPROC = CFUNCTYPE(BOOL, HDC, INT64, POINTER(INT64), POINTER(INT64), POINTER(INT64)) 	# http://www.opengl.org/registry/api/wglext.h:594
# I3D_digital_video_control (http://www.opengl.org/registry/api/wglext.h:597)
WGL_I3D_digital_video_control = 1 	# http://www.opengl.org/registry/api/wglext.h:598
# http://www.opengl.org/registry/api/wglext.h:600
wglGetDigitalVideoParametersI3D = _link_function('wglGetDigitalVideoParametersI3D', BOOL, [HDC, c_int, POINTER(c_int)], 'I3D_digital_video_control')

# http://www.opengl.org/registry/api/wglext.h:601
wglSetDigitalVideoParametersI3D = _link_function('wglSetDigitalVideoParametersI3D', BOOL, [HDC, c_int, POINTER(c_int)], 'I3D_digital_video_control')

PFNWGLGETDIGITALVIDEOPARAMETERSI3DPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:603
PFNWGLSETDIGITALVIDEOPARAMETERSI3DPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:604
# I3D_gamma (http://www.opengl.org/registry/api/wglext.h:607)
WGL_I3D_gamma = 1 	# http://www.opengl.org/registry/api/wglext.h:608
# http://www.opengl.org/registry/api/wglext.h:610
wglGetGammaTableParametersI3D = _link_function('wglGetGammaTableParametersI3D', BOOL, [HDC, c_int, POINTER(c_int)], 'I3D_gamma')

# http://www.opengl.org/registry/api/wglext.h:611
wglSetGammaTableParametersI3D = _link_function('wglSetGammaTableParametersI3D', BOOL, [HDC, c_int, POINTER(c_int)], 'I3D_gamma')

# http://www.opengl.org/registry/api/wglext.h:612
wglGetGammaTableI3D = _link_function('wglGetGammaTableI3D', BOOL, [HDC, c_int, POINTER(USHORT), POINTER(USHORT), POINTER(USHORT)], 'I3D_gamma')

# http://www.opengl.org/registry/api/wglext.h:613
wglSetGammaTableI3D = _link_function('wglSetGammaTableI3D', BOOL, [HDC, c_int, POINTER(USHORT), POINTER(USHORT), POINTER(USHORT)], 'I3D_gamma')

PFNWGLGETGAMMATABLEPARAMETERSI3DPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:615
PFNWGLSETGAMMATABLEPARAMETERSI3DPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:616
PFNWGLGETGAMMATABLEI3DPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(USHORT), POINTER(USHORT), POINTER(USHORT)) 	# http://www.opengl.org/registry/api/wglext.h:617
PFNWGLSETGAMMATABLEI3DPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(USHORT), POINTER(USHORT), POINTER(USHORT)) 	# http://www.opengl.org/registry/api/wglext.h:618
# I3D_genlock (http://www.opengl.org/registry/api/wglext.h:621)
WGL_I3D_genlock = 1 	# http://www.opengl.org/registry/api/wglext.h:622
# http://www.opengl.org/registry/api/wglext.h:624
wglEnableGenlockI3D = _link_function('wglEnableGenlockI3D', BOOL, [HDC], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:625
wglDisableGenlockI3D = _link_function('wglDisableGenlockI3D', BOOL, [HDC], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:626
wglIsEnabledGenlockI3D = _link_function('wglIsEnabledGenlockI3D', BOOL, [HDC, POINTER(BOOL)], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:627
wglGenlockSourceI3D = _link_function('wglGenlockSourceI3D', BOOL, [HDC, UINT], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:628
wglGetGenlockSourceI3D = _link_function('wglGetGenlockSourceI3D', BOOL, [HDC, POINTER(UINT)], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:629
wglGenlockSourceEdgeI3D = _link_function('wglGenlockSourceEdgeI3D', BOOL, [HDC, UINT], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:630
wglGetGenlockSourceEdgeI3D = _link_function('wglGetGenlockSourceEdgeI3D', BOOL, [HDC, POINTER(UINT)], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:631
wglGenlockSampleRateI3D = _link_function('wglGenlockSampleRateI3D', BOOL, [HDC, UINT], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:632
wglGetGenlockSampleRateI3D = _link_function('wglGetGenlockSampleRateI3D', BOOL, [HDC, POINTER(UINT)], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:633
wglGenlockSourceDelayI3D = _link_function('wglGenlockSourceDelayI3D', BOOL, [HDC, UINT], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:634
wglGetGenlockSourceDelayI3D = _link_function('wglGetGenlockSourceDelayI3D', BOOL, [HDC, POINTER(UINT)], 'I3D_genlock')

# http://www.opengl.org/registry/api/wglext.h:635
wglQueryGenlockMaxSourceDelayI3D = _link_function('wglQueryGenlockMaxSourceDelayI3D', BOOL, [HDC, POINTER(UINT), POINTER(UINT)], 'I3D_genlock')

PFNWGLENABLEGENLOCKI3DPROC = CFUNCTYPE(BOOL, HDC) 	# http://www.opengl.org/registry/api/wglext.h:637
PFNWGLDISABLEGENLOCKI3DPROC = CFUNCTYPE(BOOL, HDC) 	# http://www.opengl.org/registry/api/wglext.h:638
PFNWGLISENABLEDGENLOCKI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(BOOL)) 	# http://www.opengl.org/registry/api/wglext.h:639
PFNWGLGENLOCKSOURCEI3DPROC = CFUNCTYPE(BOOL, HDC, UINT) 	# http://www.opengl.org/registry/api/wglext.h:640
PFNWGLGETGENLOCKSOURCEI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:641
PFNWGLGENLOCKSOURCEEDGEI3DPROC = CFUNCTYPE(BOOL, HDC, UINT) 	# http://www.opengl.org/registry/api/wglext.h:642
PFNWGLGETGENLOCKSOURCEEDGEI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:643
PFNWGLGENLOCKSAMPLERATEI3DPROC = CFUNCTYPE(BOOL, HDC, UINT) 	# http://www.opengl.org/registry/api/wglext.h:644
PFNWGLGETGENLOCKSAMPLERATEI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:645
PFNWGLGENLOCKSOURCEDELAYI3DPROC = CFUNCTYPE(BOOL, HDC, UINT) 	# http://www.opengl.org/registry/api/wglext.h:646
PFNWGLGETGENLOCKSOURCEDELAYI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:647
PFNWGLQUERYGENLOCKMAXSOURCEDELAYI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(UINT), POINTER(UINT)) 	# http://www.opengl.org/registry/api/wglext.h:648
# I3D_image_buffer (http://www.opengl.org/registry/api/wglext.h:651)
WGL_I3D_image_buffer = 1 	# http://www.opengl.org/registry/api/wglext.h:652

# http://www.opengl.org/registry/api/wglext.h:654
wglCreateImageBufferI3D = _link_function('wglCreateImageBufferI3D', LPVOID, [HDC, DWORD, UINT], 'I3D_image_buffer')

# http://www.opengl.org/registry/api/wglext.h:655
wglDestroyImageBufferI3D = _link_function('wglDestroyImageBufferI3D', BOOL, [HDC, LPVOID], 'I3D_image_buffer')

# http://www.opengl.org/registry/api/wglext.h:656
wglAssociateImageBufferEventsI3D = _link_function('wglAssociateImageBufferEventsI3D', BOOL, [HDC, POINTER(HANDLE), POINTER(LPVOID), POINTER(DWORD), UINT], 'I3D_image_buffer')

# http://www.opengl.org/registry/api/wglext.h:657
wglReleaseImageBufferEventsI3D = _link_function('wglReleaseImageBufferEventsI3D', BOOL, [HDC, POINTER(LPVOID), UINT], 'I3D_image_buffer')

PFNWGLCREATEIMAGEBUFFERI3DPROC = CFUNCTYPE(LPVOID, HDC, DWORD, UINT) 	# http://www.opengl.org/registry/api/wglext.h:659
PFNWGLDESTROYIMAGEBUFFERI3DPROC = CFUNCTYPE(BOOL, HDC, LPVOID) 	# http://www.opengl.org/registry/api/wglext.h:660
PFNWGLASSOCIATEIMAGEBUFFEREVENTSI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(HANDLE), POINTER(LPVOID), POINTER(DWORD), UINT) 	# http://www.opengl.org/registry/api/wglext.h:661
PFNWGLRELEASEIMAGEBUFFEREVENTSI3DPROC = CFUNCTYPE(BOOL, HDC, POINTER(LPVOID), UINT) 	# http://www.opengl.org/registry/api/wglext.h:662
# I3D_swap_frame_lock (http://www.opengl.org/registry/api/wglext.h:665)
WGL_I3D_swap_frame_lock = 1 	# http://www.opengl.org/registry/api/wglext.h:666
# http://www.opengl.org/registry/api/wglext.h:668
wglEnableFrameLockI3D = _link_function('wglEnableFrameLockI3D', BOOL, [], 'I3D_swap_frame_lock')

# http://www.opengl.org/registry/api/wglext.h:669
wglDisableFrameLockI3D = _link_function('wglDisableFrameLockI3D', BOOL, [], 'I3D_swap_frame_lock')

# http://www.opengl.org/registry/api/wglext.h:670
wglIsEnabledFrameLockI3D = _link_function('wglIsEnabledFrameLockI3D', BOOL, [POINTER(BOOL)], 'I3D_swap_frame_lock')

# http://www.opengl.org/registry/api/wglext.h:671
wglQueryFrameLockMasterI3D = _link_function('wglQueryFrameLockMasterI3D', BOOL, [POINTER(BOOL)], 'I3D_swap_frame_lock')

PFNWGLENABLEFRAMELOCKI3DPROC = CFUNCTYPE(BOOL) 	# http://www.opengl.org/registry/api/wglext.h:673
PFNWGLDISABLEFRAMELOCKI3DPROC = CFUNCTYPE(BOOL) 	# http://www.opengl.org/registry/api/wglext.h:674
PFNWGLISENABLEDFRAMELOCKI3DPROC = CFUNCTYPE(BOOL, POINTER(BOOL)) 	# http://www.opengl.org/registry/api/wglext.h:675
PFNWGLQUERYFRAMELOCKMASTERI3DPROC = CFUNCTYPE(BOOL, POINTER(BOOL)) 	# http://www.opengl.org/registry/api/wglext.h:676
# I3D_swap_frame_usage (http://www.opengl.org/registry/api/wglext.h:679)
WGL_I3D_swap_frame_usage = 1 	# http://www.opengl.org/registry/api/wglext.h:680
# http://www.opengl.org/registry/api/wglext.h:682
wglGetFrameUsageI3D = _link_function('wglGetFrameUsageI3D', BOOL, [POINTER(c_float)], 'I3D_swap_frame_usage')

# http://www.opengl.org/registry/api/wglext.h:683
wglBeginFrameTrackingI3D = _link_function('wglBeginFrameTrackingI3D', BOOL, [], 'I3D_swap_frame_usage')

# http://www.opengl.org/registry/api/wglext.h:684
wglEndFrameTrackingI3D = _link_function('wglEndFrameTrackingI3D', BOOL, [], 'I3D_swap_frame_usage')

# http://www.opengl.org/registry/api/wglext.h:685
wglQueryFrameTrackingI3D = _link_function('wglQueryFrameTrackingI3D', BOOL, [POINTER(DWORD), POINTER(DWORD), POINTER(c_float)], 'I3D_swap_frame_usage')

PFNWGLGETFRAMEUSAGEI3DPROC = CFUNCTYPE(BOOL, POINTER(c_float)) 	# http://www.opengl.org/registry/api/wglext.h:687
PFNWGLBEGINFRAMETRACKINGI3DPROC = CFUNCTYPE(BOOL) 	# http://www.opengl.org/registry/api/wglext.h:688
PFNWGLENDFRAMETRACKINGI3DPROC = CFUNCTYPE(BOOL) 	# http://www.opengl.org/registry/api/wglext.h:689
PFNWGLQUERYFRAMETRACKINGI3DPROC = CFUNCTYPE(BOOL, POINTER(DWORD), POINTER(DWORD), POINTER(c_float)) 	# http://www.opengl.org/registry/api/wglext.h:690
# ATI_pixel_format_float (http://www.opengl.org/registry/api/wglext.h:693)
WGL_ATI_pixel_format_float = 1 	# http://www.opengl.org/registry/api/wglext.h:694
# NV_float_buffer (http://www.opengl.org/registry/api/wglext.h:697)
WGL_NV_float_buffer = 1 	# http://www.opengl.org/registry/api/wglext.h:698
# EXT_pixel_format_packed_float (http://www.opengl.org/registry/api/wglext.h:701)
WGL_EXT_pixel_format_packed_float = 1 	# http://www.opengl.org/registry/api/wglext.h:702
# EXT_framebuffer_sRGB (http://www.opengl.org/registry/api/wglext.h:705)
WGL_EXT_framebuffer_sRGB = 1 	# http://www.opengl.org/registry/api/wglext.h:706
# NV_present_video (http://www.opengl.org/registry/api/wglext.h:709)
WGL_NV_present_video = 1 	# http://www.opengl.org/registry/api/wglext.h:710
# http://www.opengl.org/registry/api/wglext.h:712
wglEnumerateVideoDevicesNV = _link_function('wglEnumerateVideoDevicesNV', c_int, [HDC, POINTER(HVIDEOOUTPUTDEVICENV)], 'NV_present_video')

# http://www.opengl.org/registry/api/wglext.h:713
wglBindVideoDeviceNV = _link_function('wglBindVideoDeviceNV', BOOL, [HDC, c_uint, HVIDEOOUTPUTDEVICENV, POINTER(c_int)], 'NV_present_video')

# http://www.opengl.org/registry/api/wglext.h:714
wglQueryCurrentContextNV = _link_function('wglQueryCurrentContextNV', BOOL, [c_int, POINTER(c_int)], 'NV_present_video')

PFNWGLENUMERATEVIDEODEVICESNVPROC = CFUNCTYPE(c_int, HDC, POINTER(HVIDEOOUTPUTDEVICENV)) 	# http://www.opengl.org/registry/api/wglext.h:716
PFNWGLBINDVIDEODEVICENVPROC = CFUNCTYPE(BOOL, HDC, c_uint, HVIDEOOUTPUTDEVICENV, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:717
PFNWGLQUERYCURRENTCONTEXTNVPROC = CFUNCTYPE(BOOL, c_int, POINTER(c_int)) 	# http://www.opengl.org/registry/api/wglext.h:718
# NV_video_out (http://www.opengl.org/registry/api/wglext.h:721)
WGL_NV_video_out = 1 	# http://www.opengl.org/registry/api/wglext.h:722
# http://www.opengl.org/registry/api/wglext.h:724
wglGetVideoDeviceNV = _link_function('wglGetVideoDeviceNV', BOOL, [HDC, c_int, POINTER(HPVIDEODEV)], 'NV_video_out')

# http://www.opengl.org/registry/api/wglext.h:725
wglReleaseVideoDeviceNV = _link_function('wglReleaseVideoDeviceNV', BOOL, [HPVIDEODEV], 'NV_video_out')

# http://www.opengl.org/registry/api/wglext.h:726
wglBindVideoImageNV = _link_function('wglBindVideoImageNV', BOOL, [HPVIDEODEV, HPBUFFERARB, c_int], 'NV_video_out')

# http://www.opengl.org/registry/api/wglext.h:727
wglReleaseVideoImageNV = _link_function('wglReleaseVideoImageNV', BOOL, [HPBUFFERARB, c_int], 'NV_video_out')

# http://www.opengl.org/registry/api/wglext.h:728
wglSendPbufferToVideoNV = _link_function('wglSendPbufferToVideoNV', BOOL, [HPBUFFERARB, c_int, POINTER(c_ulong), BOOL], 'NV_video_out')

# http://www.opengl.org/registry/api/wglext.h:729
wglGetVideoInfoNV = _link_function('wglGetVideoInfoNV', BOOL, [HPVIDEODEV, POINTER(c_ulong), POINTER(c_ulong)], 'NV_video_out')

PFNWGLGETVIDEODEVICENVPROC = CFUNCTYPE(BOOL, HDC, c_int, POINTER(HPVIDEODEV)) 	# http://www.opengl.org/registry/api/wglext.h:731
PFNWGLRELEASEVIDEODEVICENVPROC = CFUNCTYPE(BOOL, HPVIDEODEV) 	# http://www.opengl.org/registry/api/wglext.h:732
PFNWGLBINDVIDEOIMAGENVPROC = CFUNCTYPE(BOOL, HPVIDEODEV, HPBUFFERARB, c_int) 	# http://www.opengl.org/registry/api/wglext.h:733
PFNWGLRELEASEVIDEOIMAGENVPROC = CFUNCTYPE(BOOL, HPBUFFERARB, c_int) 	# http://www.opengl.org/registry/api/wglext.h:734
PFNWGLSENDPBUFFERTOVIDEONVPROC = CFUNCTYPE(BOOL, HPBUFFERARB, c_int, POINTER(c_ulong), BOOL) 	# http://www.opengl.org/registry/api/wglext.h:735
PFNWGLGETVIDEOINFONVPROC = CFUNCTYPE(BOOL, HPVIDEODEV, POINTER(c_ulong), POINTER(c_ulong)) 	# http://www.opengl.org/registry/api/wglext.h:736
# NV_swap_group (http://www.opengl.org/registry/api/wglext.h:739)
WGL_NV_swap_group = 1 	# http://www.opengl.org/registry/api/wglext.h:740
# http://www.opengl.org/registry/api/wglext.h:742
wglJoinSwapGroupNV = _link_function('wglJoinSwapGroupNV', BOOL, [HDC, GLuint], 'NV_swap_group')

# http://www.opengl.org/registry/api/wglext.h:743
wglBindSwapBarrierNV = _link_function('wglBindSwapBarrierNV', BOOL, [GLuint, GLuint], 'NV_swap_group')

# http://www.opengl.org/registry/api/wglext.h:744
wglQuerySwapGroupNV = _link_function('wglQuerySwapGroupNV', BOOL, [HDC, POINTER(GLuint), POINTER(GLuint)], 'NV_swap_group')

# http://www.opengl.org/registry/api/wglext.h:745
wglQueryMaxSwapGroupsNV = _link_function('wglQueryMaxSwapGroupsNV', BOOL, [HDC, POINTER(GLuint), POINTER(GLuint)], 'NV_swap_group')

# http://www.opengl.org/registry/api/wglext.h:746
wglQueryFrameCountNV = _link_function('wglQueryFrameCountNV', BOOL, [HDC, POINTER(GLuint)], 'NV_swap_group')

# http://www.opengl.org/registry/api/wglext.h:747
wglResetFrameCountNV = _link_function('wglResetFrameCountNV', BOOL, [HDC], 'NV_swap_group')

PFNWGLJOINSWAPGROUPNVPROC = CFUNCTYPE(BOOL, HDC, GLuint) 	# http://www.opengl.org/registry/api/wglext.h:749
PFNWGLBINDSWAPBARRIERNVPROC = CFUNCTYPE(BOOL, GLuint, GLuint) 	# http://www.opengl.org/registry/api/wglext.h:750
PFNWGLQUERYSWAPGROUPNVPROC = CFUNCTYPE(BOOL, HDC, POINTER(GLuint), POINTER(GLuint)) 	# http://www.opengl.org/registry/api/wglext.h:751
PFNWGLQUERYMAXSWAPGROUPSNVPROC = CFUNCTYPE(BOOL, HDC, POINTER(GLuint), POINTER(GLuint)) 	# http://www.opengl.org/registry/api/wglext.h:752
PFNWGLQUERYFRAMECOUNTNVPROC = CFUNCTYPE(BOOL, HDC, POINTER(GLuint)) 	# http://www.opengl.org/registry/api/wglext.h:753
PFNWGLRESETFRAMECOUNTNVPROC = CFUNCTYPE(BOOL, HDC) 	# http://www.opengl.org/registry/api/wglext.h:754
# NV_gpu_affinity (http://www.opengl.org/registry/api/wglext.h:757)
WGL_NV_gpu_affinity = 1 	# http://www.opengl.org/registry/api/wglext.h:758
# http://www.opengl.org/registry/api/wglext.h:760
wglEnumGpusNV = _link_function('wglEnumGpusNV', BOOL, [UINT, POINTER(HGPUNV)], 'NV_gpu_affinity')

# http://www.opengl.org/registry/api/wglext.h:761
wglEnumGpuDevicesNV = _link_function('wglEnumGpuDevicesNV', BOOL, [HGPUNV, UINT, PGPU_DEVICE], 'NV_gpu_affinity')

# http://www.opengl.org/registry/api/wglext.h:762
wglCreateAffinityDCNV = _link_function('wglCreateAffinityDCNV', HDC, [POINTER(HGPUNV)], 'NV_gpu_affinity')

# http://www.opengl.org/registry/api/wglext.h:763
wglEnumGpusFromAffinityDCNV = _link_function('wglEnumGpusFromAffinityDCNV', BOOL, [HDC, UINT, POINTER(HGPUNV)], 'NV_gpu_affinity')

# http://www.opengl.org/registry/api/wglext.h:764
wglDeleteDCNV = _link_function('wglDeleteDCNV', BOOL, [HDC], 'NV_gpu_affinity')

PFNWGLENUMGPUSNVPROC = CFUNCTYPE(BOOL, UINT, POINTER(HGPUNV)) 	# http://www.opengl.org/registry/api/wglext.h:766
PFNWGLENUMGPUDEVICESNVPROC = CFUNCTYPE(BOOL, HGPUNV, UINT, PGPU_DEVICE) 	# http://www.opengl.org/registry/api/wglext.h:767
PFNWGLCREATEAFFINITYDCNVPROC = CFUNCTYPE(HDC, POINTER(HGPUNV)) 	# http://www.opengl.org/registry/api/wglext.h:768
PFNWGLENUMGPUSFROMAFFINITYDCNVPROC = CFUNCTYPE(BOOL, HDC, UINT, POINTER(HGPUNV)) 	# http://www.opengl.org/registry/api/wglext.h:769
PFNWGLDELETEDCNVPROC = CFUNCTYPE(BOOL, HDC) 	# http://www.opengl.org/registry/api/wglext.h:770

__all__ = [
    'ERROR_INCOMPATIBLE_DEVICE_CONTEXTS_ARB',
    'ERROR_INVALID_PIXEL_TYPE_ARB',
    'ERROR_INVALID_PIXEL_TYPE_EXT',
    'ERROR_INVALID_VERSION_ARB',
    'GPU_DEVICE',
    'HGPUNV',
    'HPBUFFERARB',
    'HPBUFFEREXT',
    'HPGPUNV',
    'HPVIDEODEV',
    'HVIDEOOUTPUTDEVICENV',
    'PFNWGLALLOCATEMEMORYNVPROC',
    'PFNWGLASSOCIATEIMAGEBUFFEREVENTSI3DPROC',
    'PFNWGLBEGINFRAMETRACKINGI3DPROC',
    'PFNWGLBINDDISPLAYCOLORTABLEEXTPROC',
    'PFNWGLBINDSWAPBARRIERNVPROC',
    'PFNWGLBINDTEXIMAGEARBPROC',
    'PFNWGLBINDVIDEODEVICENVPROC',
    'PFNWGLBINDVIDEOIMAGENVPROC',
    'PFNWGLCHOOSEPIXELFORMATARBPROC',
    'PFNWGLCHOOSEPIXELFORMATEXTPROC',
    'PFNWGLCREATEAFFINITYDCNVPROC',
    'PFNWGLCREATEBUFFERREGIONARBPROC',
    'PFNWGLCREATECONTEXTATTRIBSARBPROC',
    'PFNWGLCREATEDISPLAYCOLORTABLEEXTPROC',
    'PFNWGLCREATEIMAGEBUFFERI3DPROC',
    'PFNWGLCREATEPBUFFERARBPROC',
    'PFNWGLCREATEPBUFFEREXTPROC',
    'PFNWGLDELETEBUFFERREGIONARBPROC',
    'PFNWGLDELETEDCNVPROC',
    'PFNWGLDESTROYDISPLAYCOLORTABLEEXTPROC',
    'PFNWGLDESTROYIMAGEBUFFERI3DPROC',
    'PFNWGLDESTROYPBUFFERARBPROC',
    'PFNWGLDESTROYPBUFFEREXTPROC',
    'PFNWGLDISABLEFRAMELOCKI3DPROC',
    'PFNWGLDISABLEGENLOCKI3DPROC',
    'PFNWGLENABLEFRAMELOCKI3DPROC',
    'PFNWGLENABLEGENLOCKI3DPROC',
    'PFNWGLENDFRAMETRACKINGI3DPROC',
    'PFNWGLENUMERATEVIDEODEVICESNVPROC',
    'PFNWGLENUMGPUDEVICESNVPROC',
    'PFNWGLENUMGPUSFROMAFFINITYDCNVPROC',
    'PFNWGLENUMGPUSNVPROC',
    'PFNWGLFREEMEMORYNVPROC',
    'PFNWGLGENLOCKSAMPLERATEI3DPROC',
    'PFNWGLGENLOCKSOURCEDELAYI3DPROC',
    'PFNWGLGENLOCKSOURCEEDGEI3DPROC',
    'PFNWGLGENLOCKSOURCEI3DPROC',
    'PFNWGLGETCURRENTREADDCARBPROC',
    'PFNWGLGETCURRENTREADDCEXTPROC',
    'PFNWGLGETDIGITALVIDEOPARAMETERSI3DPROC',
    'PFNWGLGETEXTENSIONSSTRINGARBPROC',
    'PFNWGLGETEXTENSIONSSTRINGEXTPROC',
    'PFNWGLGETFRAMEUSAGEI3DPROC',
    'PFNWGLGETGAMMATABLEI3DPROC',
    'PFNWGLGETGAMMATABLEPARAMETERSI3DPROC',
    'PFNWGLGETGENLOCKSAMPLERATEI3DPROC',
    'PFNWGLGETGENLOCKSOURCEDELAYI3DPROC',
    'PFNWGLGETGENLOCKSOURCEEDGEI3DPROC',
    'PFNWGLGETGENLOCKSOURCEI3DPROC',
    'PFNWGLGETMSCRATEOMLPROC',
    'PFNWGLGETPBUFFERDCARBPROC',
    'PFNWGLGETPBUFFERDCEXTPROC',
    'PFNWGLGETPIXELFORMATATTRIBFVARBPROC',
    'PFNWGLGETPIXELFORMATATTRIBFVEXTPROC',
    'PFNWGLGETPIXELFORMATATTRIBIVARBPROC',
    'PFNWGLGETPIXELFORMATATTRIBIVEXTPROC',
    'PFNWGLGETSWAPINTERVALEXTPROC',
    'PFNWGLGETSYNCVALUESOMLPROC',
    'PFNWGLGETVIDEODEVICENVPROC',
    'PFNWGLGETVIDEOINFONVPROC',
    'PFNWGLISENABLEDFRAMELOCKI3DPROC',
    'PFNWGLISENABLEDGENLOCKI3DPROC',
    'PFNWGLJOINSWAPGROUPNVPROC',
    'PFNWGLLOADDISPLAYCOLORTABLEEXTPROC',
    'PFNWGLMAKECONTEXTCURRENTARBPROC',
    'PFNWGLMAKECONTEXTCURRENTEXTPROC',
    'PFNWGLQUERYCURRENTCONTEXTNVPROC',
    'PFNWGLQUERYFRAMECOUNTNVPROC',
    'PFNWGLQUERYFRAMELOCKMASTERI3DPROC',
    'PFNWGLQUERYFRAMETRACKINGI3DPROC',
    'PFNWGLQUERYGENLOCKMAXSOURCEDELAYI3DPROC',
    'PFNWGLQUERYMAXSWAPGROUPSNVPROC',
    'PFNWGLQUERYPBUFFERARBPROC',
    'PFNWGLQUERYPBUFFEREXTPROC',
    'PFNWGLQUERYSWAPGROUPNVPROC',
    'PFNWGLRELEASEIMAGEBUFFEREVENTSI3DPROC',
    'PFNWGLRELEASEPBUFFERDCARBPROC',
    'PFNWGLRELEASEPBUFFERDCEXTPROC',
    'PFNWGLRELEASETEXIMAGEARBPROC',
    'PFNWGLRELEASEVIDEODEVICENVPROC',
    'PFNWGLRELEASEVIDEOIMAGENVPROC',
    'PFNWGLRESETFRAMECOUNTNVPROC',
    'PFNWGLRESTOREBUFFERREGIONARBPROC',
    'PFNWGLSAVEBUFFERREGIONARBPROC',
    'PFNWGLSENDPBUFFERTOVIDEONVPROC',
    'PFNWGLSETDIGITALVIDEOPARAMETERSI3DPROC',
    'PFNWGLSETGAMMATABLEI3DPROC',
    'PFNWGLSETGAMMATABLEPARAMETERSI3DPROC',
    'PFNWGLSETPBUFFERATTRIBARBPROC',
    'PFNWGLSWAPBUFFERSMSCOMLPROC',
    'PFNWGLSWAPINTERVALEXTPROC',
    'PFNWGLSWAPLAYERBUFFERSMSCOMLPROC',
    'PFNWGLWAITFORMSCOMLPROC',
    'PFNWGLWAITFORSBCOMLPROC',
    'PGPU_DEVICE',
    'WGL_ACCELERATION_ARB',
    'WGL_ACCELERATION_EXT',
    'WGL_ACCUM_ALPHA_BITS_ARB',
    'WGL_ACCUM_ALPHA_BITS_EXT',
    'WGL_ACCUM_BITS_ARB',
    'WGL_ACCUM_BITS_EXT',
    'WGL_ACCUM_BLUE_BITS_ARB',
    'WGL_ACCUM_BLUE_BITS_EXT',
    'WGL_ACCUM_GREEN_BITS_ARB',
    'WGL_ACCUM_GREEN_BITS_EXT',
    'WGL_ACCUM_RED_BITS_ARB',
    'WGL_ACCUM_RED_BITS_EXT',
    'WGL_ALPHA_BITS_ARB',
    'WGL_ALPHA_BITS_EXT',
    'WGL_ALPHA_SHIFT_ARB',
    'WGL_ALPHA_SHIFT_EXT',
    'WGL_AUX0_ARB',
    'WGL_AUX1_ARB',
    'WGL_AUX2_ARB',
    'WGL_AUX3_ARB',
    'WGL_AUX4_ARB',
    'WGL_AUX5_ARB',
    'WGL_AUX6_ARB',
    'WGL_AUX7_ARB',
    'WGL_AUX8_ARB',
    'WGL_AUX9_ARB',
    'WGL_AUX_BUFFERS_ARB',
    'WGL_AUX_BUFFERS_EXT',
    'WGL_BACK_COLOR_BUFFER_BIT_ARB',
    'WGL_BACK_LEFT_ARB',
    'WGL_BACK_RIGHT_ARB',
    'WGL_BIND_TO_TEXTURE_DEPTH_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_DEPTH_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RGBA_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RGB_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RG_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_R_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_RGBA_NV',
    'WGL_BIND_TO_TEXTURE_RECTANGLE_RGB_NV',
    'WGL_BIND_TO_TEXTURE_RGBA_ARB',
    'WGL_BIND_TO_TEXTURE_RGB_ARB',
    'WGL_BIND_TO_VIDEO_RGBA_NV',
    'WGL_BIND_TO_VIDEO_RGB_AND_DEPTH_NV',
    'WGL_BIND_TO_VIDEO_RGB_NV',
    'WGL_BLUE_BITS_ARB',
    'WGL_BLUE_BITS_EXT',
    'WGL_BLUE_SHIFT_ARB',
    'WGL_BLUE_SHIFT_EXT',
    'WGL_COLOR_BITS_ARB',
    'WGL_COLOR_BITS_EXT',
    'WGL_CONTEXT_DEBUG_BIT_ARB',
    'WGL_CONTEXT_FLAGS_ARB',
    'WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB',
    'WGL_CONTEXT_LAYER_PLANE_ARB',
    'WGL_CONTEXT_MAJOR_VERSION_ARB',
    'WGL_CONTEXT_MINOR_VERSION_ARB',
    'WGL_CUBE_MAP_FACE_ARB',
    'WGL_DEPTH_BITS_ARB',
    'WGL_DEPTH_BITS_EXT',
    'WGL_DEPTH_BUFFER_BIT_ARB',
    'WGL_DEPTH_COMPONENT_NV',
    'WGL_DEPTH_FLOAT_EXT',
    'WGL_DEPTH_TEXTURE_FORMAT_NV',
    'WGL_DIGITAL_VIDEO_CURSOR_ALPHA_FRAMEBUFFER_I3D',
    'WGL_DIGITAL_VIDEO_CURSOR_ALPHA_VALUE_I3D',
    'WGL_DIGITAL_VIDEO_CURSOR_INCLUDED_I3D',
    'WGL_DIGITAL_VIDEO_GAMMA_CORRECTED_I3D',
    'WGL_DOUBLE_BUFFER_ARB',
    'WGL_DOUBLE_BUFFER_EXT',
    'WGL_DRAW_TO_BITMAP_ARB',
    'WGL_DRAW_TO_BITMAP_EXT',
    'WGL_DRAW_TO_PBUFFER_ARB',
    'WGL_DRAW_TO_PBUFFER_EXT',
    'WGL_DRAW_TO_WINDOW_ARB',
    'WGL_DRAW_TO_WINDOW_EXT',
    'WGL_ERROR_INCOMPATIBLE_AFFINITY_MASKS_NV',
    'WGL_ERROR_MISSING_AFFINITY_MASK_NV',
    'WGL_FLOAT_COMPONENTS_NV',
    'WGL_FRAMEBUFFER_SRGB_CAPABLE_EXT',
    'WGL_FRONT_COLOR_BUFFER_BIT_ARB',
    'WGL_FRONT_LEFT_ARB',
    'WGL_FRONT_RIGHT_ARB',
    'WGL_FULL_ACCELERATION_ARB',
    'WGL_FULL_ACCELERATION_EXT',
    'WGL_GAMMA_EXCLUDE_DESKTOP_I3D',
    'WGL_GAMMA_TABLE_SIZE_I3D',
    'WGL_GENERIC_ACCELERATION_ARB',
    'WGL_GENERIC_ACCELERATION_EXT',
    'WGL_GENLOCK_SOURCE_DIGITAL_FIELD_I3D',
    'WGL_GENLOCK_SOURCE_DIGITAL_SYNC_I3D',
    'WGL_GENLOCK_SOURCE_EDGE_BOTH_I3D',
    'WGL_GENLOCK_SOURCE_EDGE_FALLING_I3D',
    'WGL_GENLOCK_SOURCE_EDGE_RISING_I3D',
    'WGL_GENLOCK_SOURCE_EXTENAL_FIELD_I3D',
    'WGL_GENLOCK_SOURCE_EXTENAL_SYNC_I3D',
    'WGL_GENLOCK_SOURCE_EXTENAL_TTL_I3D',
    'WGL_GENLOCK_SOURCE_MULTIVIEW_I3D',
    'WGL_GREEN_BITS_ARB',
    'WGL_GREEN_BITS_EXT',
    'WGL_GREEN_SHIFT_ARB',
    'WGL_GREEN_SHIFT_EXT',
    'WGL_IMAGE_BUFFER_LOCK_I3D',
    'WGL_IMAGE_BUFFER_MIN_ACCESS_I3D',
    'WGL_MAX_PBUFFER_HEIGHT_ARB',
    'WGL_MAX_PBUFFER_HEIGHT_EXT',
    'WGL_MAX_PBUFFER_PIXELS_ARB',
    'WGL_MAX_PBUFFER_PIXELS_EXT',
    'WGL_MAX_PBUFFER_WIDTH_ARB',
    'WGL_MAX_PBUFFER_WIDTH_EXT',
    'WGL_MIPMAP_LEVEL_ARB',
    'WGL_MIPMAP_TEXTURE_ARB',
    'WGL_NEED_PALETTE_ARB',
    'WGL_NEED_PALETTE_EXT',
    'WGL_NEED_SYSTEM_PALETTE_ARB',
    'WGL_NEED_SYSTEM_PALETTE_EXT',
    'WGL_NO_ACCELERATION_ARB',
    'WGL_NO_ACCELERATION_EXT',
    'WGL_NO_TEXTURE_ARB',
    'WGL_NUMBER_OVERLAYS_ARB',
    'WGL_NUMBER_OVERLAYS_EXT',
    'WGL_NUMBER_PIXEL_FORMATS_ARB',
    'WGL_NUMBER_PIXEL_FORMATS_EXT',
    'WGL_NUMBER_UNDERLAYS_ARB',
    'WGL_NUMBER_UNDERLAYS_EXT',
    'WGL_NUM_VIDEO_SLOTS_NV',
    'WGL_OPTIMAL_PBUFFER_HEIGHT_EXT',
    'WGL_OPTIMAL_PBUFFER_WIDTH_EXT',
    'WGL_PBUFFER_HEIGHT_ARB',
    'WGL_PBUFFER_HEIGHT_EXT',
    'WGL_PBUFFER_LARGEST_ARB',
    'WGL_PBUFFER_LARGEST_EXT',
    'WGL_PBUFFER_LOST_ARB',
    'WGL_PBUFFER_WIDTH_ARB',
    'WGL_PBUFFER_WIDTH_EXT',
    'WGL_PIXEL_TYPE_ARB',
    'WGL_PIXEL_TYPE_EXT',
    'WGL_RED_BITS_ARB',
    'WGL_RED_BITS_EXT',
    'WGL_RED_SHIFT_ARB',
    'WGL_RED_SHIFT_EXT',
    'WGL_SAMPLES_3DFX',
    'WGL_SAMPLES_ARB',
    'WGL_SAMPLES_EXT',
    'WGL_SAMPLE_BUFFERS_3DFX',
    'WGL_SAMPLE_BUFFERS_ARB',
    'WGL_SAMPLE_BUFFERS_EXT',
    'WGL_SHARE_ACCUM_ARB',
    'WGL_SHARE_ACCUM_EXT',
    'WGL_SHARE_DEPTH_ARB',
    'WGL_SHARE_DEPTH_EXT',
    'WGL_SHARE_STENCIL_ARB',
    'WGL_SHARE_STENCIL_EXT',
    'WGL_STENCIL_BITS_ARB',
    'WGL_STENCIL_BITS_EXT',
    'WGL_STENCIL_BUFFER_BIT_ARB',
    'WGL_STEREO_ARB',
    'WGL_STEREO_EMITTER_DISABLE_3DL',
    'WGL_STEREO_EMITTER_ENABLE_3DL',
    'WGL_STEREO_EXT',
    'WGL_STEREO_POLARITY_INVERT_3DL',
    'WGL_STEREO_POLARITY_NORMAL_3DL',
    'WGL_SUPPORT_GDI_ARB',
    'WGL_SUPPORT_GDI_EXT',
    'WGL_SUPPORT_OPENGL_ARB',
    'WGL_SUPPORT_OPENGL_EXT',
    'WGL_SWAP_COPY_ARB',
    'WGL_SWAP_COPY_EXT',
    'WGL_SWAP_EXCHANGE_ARB',
    'WGL_SWAP_EXCHANGE_EXT',
    'WGL_SWAP_LAYER_BUFFERS_ARB',
    'WGL_SWAP_LAYER_BUFFERS_EXT',
    'WGL_SWAP_METHOD_ARB',
    'WGL_SWAP_METHOD_EXT',
    'WGL_SWAP_UNDEFINED_ARB',
    'WGL_SWAP_UNDEFINED_EXT',
    'WGL_TEXTURE_1D_ARB',
    'WGL_TEXTURE_2D_ARB',
    'WGL_TEXTURE_CUBE_MAP_ARB',
    'WGL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB',
    'WGL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB',
    'WGL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB',
    'WGL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB',
    'WGL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB',
    'WGL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB',
    'WGL_TEXTURE_DEPTH_COMPONENT_NV',
    'WGL_TEXTURE_FLOAT_RGBA_NV',
    'WGL_TEXTURE_FLOAT_RGB_NV',
    'WGL_TEXTURE_FLOAT_RG_NV',
    'WGL_TEXTURE_FLOAT_R_NV',
    'WGL_TEXTURE_FORMAT_ARB',
    'WGL_TEXTURE_RECTANGLE_NV',
    'WGL_TEXTURE_RGBA_ARB',
    'WGL_TEXTURE_RGB_ARB',
    'WGL_TEXTURE_TARGET_ARB',
    'WGL_TRANSPARENT_ALPHA_VALUE_ARB',
    'WGL_TRANSPARENT_ARB',
    'WGL_TRANSPARENT_BLUE_VALUE_ARB',
    'WGL_TRANSPARENT_EXT',
    'WGL_TRANSPARENT_GREEN_VALUE_ARB',
    'WGL_TRANSPARENT_INDEX_VALUE_ARB',
    'WGL_TRANSPARENT_RED_VALUE_ARB',
    'WGL_TRANSPARENT_VALUE_EXT',
    'WGL_TYPE_COLORINDEX_ARB',
    'WGL_TYPE_COLORINDEX_EXT',
    'WGL_TYPE_RGBA_ARB',
    'WGL_TYPE_RGBA_EXT',
    'WGL_TYPE_RGBA_FLOAT_ARB',
    'WGL_TYPE_RGBA_FLOAT_ATI',
    'WGL_TYPE_RGBA_UNSIGNED_FLOAT_EXT',
    'WGL_VIDEO_OUT_ALPHA_NV',
    'WGL_VIDEO_OUT_COLOR_AND_ALPHA_NV',
    'WGL_VIDEO_OUT_COLOR_AND_DEPTH_NV',
    'WGL_VIDEO_OUT_COLOR_NV',
    'WGL_VIDEO_OUT_DEPTH_NV',
    'WGL_VIDEO_OUT_FIELD_1',
    'WGL_VIDEO_OUT_FIELD_2',
    'WGL_VIDEO_OUT_FRAME',
    'WGL_VIDEO_OUT_STACKED_FIELDS_1_2',
    'WGL_VIDEO_OUT_STACKED_FIELDS_2_1',
    'WGL_WGLEXT_VERSION',
    'WIN32_LEAN_AND_MEAN',
    'WGL_3DFX_multisample',
    'WGL_ARB_buffer_region',
    'WGL_ARB_create_context',
    'WGL_ARB_extensions_string',
    'WGL_ARB_make_current_read',
    'WGL_ARB_multisample',
    'WGL_ARB_pbuffer',
    'WGL_ARB_pixel_format',
    'WGL_ARB_pixel_format_float',
    'WGL_ARB_render_texture',
    'WGL_ATI_pixel_format_float',
    'WGL_EXT_depth_float',
    'WGL_EXT_display_color_table',
    'WGL_EXT_extensions_string',
    'WGL_EXT_framebuffer_sRGB',
    'WGL_EXT_make_current_read',
    'WGL_EXT_multisample',
    'WGL_EXT_pbuffer',
    'WGL_EXT_pixel_format',
    'WGL_EXT_pixel_format_packed_float',
    'WGL_EXT_swap_control',
    'WGL_I3D_digital_video_control',
    'WGL_I3D_gamma',
    'WGL_I3D_genlock',
    'WGL_I3D_image_buffer',
    'WGL_I3D_swap_frame_lock',
    'WGL_I3D_swap_frame_usage',
    'WGL_NV_float_buffer',
    'WGL_NV_gpu_affinity',
    'WGL_NV_present_video',
    'WGL_NV_swap_group',
    'WGL_NV_vertex_array_range',
    'WGL_NV_video_out',
    'WGL_OML_sync_control',
    'wglAllocateMemoryNV',
    'wglAssociateImageBufferEventsI3D',
    'wglBeginFrameTrackingI3D',
    'wglBindDisplayColorTableEXT',
    'wglBindSwapBarrierNV',
    'wglBindTexImageARB',
    'wglBindVideoDeviceNV',
    'wglBindVideoImageNV',
    'wglChoosePixelFormatARB',
    'wglChoosePixelFormatEXT',
    'wglCreateAffinityDCNV',
    'wglCreateBufferRegionARB',
    'wglCreateContextAttribsARB',
    'wglCreateDisplayColorTableEXT',
    'wglCreateImageBufferI3D',
    'wglCreatePbufferARB',
    'wglCreatePbufferEXT',
    'wglDeleteBufferRegionARB',
    'wglDeleteDCNV',
    'wglDestroyDisplayColorTableEXT',
    'wglDestroyImageBufferI3D',
    'wglDestroyPbufferARB',
    'wglDestroyPbufferEXT',
    'wglDisableFrameLockI3D',
    'wglDisableGenlockI3D',
    'wglEnableFrameLockI3D',
    'wglEnableGenlockI3D',
    'wglEndFrameTrackingI3D',
    'wglEnumGpuDevicesNV',
    'wglEnumGpusFromAffinityDCNV',
    'wglEnumGpusNV',
    'wglEnumerateVideoDevicesNV',
    'wglFreeMemoryNV',
    'wglGenlockSampleRateI3D',
    'wglGenlockSourceDelayI3D',
    'wglGenlockSourceEdgeI3D',
    'wglGenlockSourceI3D',
    'wglGetCurrentReadDCARB',
    'wglGetCurrentReadDCEXT',
    'wglGetDigitalVideoParametersI3D',
    'wglGetExtensionsStringARB',
    'wglGetExtensionsStringEXT',
    'wglGetFrameUsageI3D',
    'wglGetGammaTableI3D',
    'wglGetGammaTableParametersI3D',
    'wglGetGenlockSampleRateI3D',
    'wglGetGenlockSourceDelayI3D',
    'wglGetGenlockSourceEdgeI3D',
    'wglGetGenlockSourceI3D',
    'wglGetMscRateOML',
    'wglGetPbufferDCARB',
    'wglGetPbufferDCEXT',
    'wglGetPixelFormatAttribfvARB',
    'wglGetPixelFormatAttribfvEXT',
    'wglGetPixelFormatAttribivARB',
    'wglGetPixelFormatAttribivEXT',
    'wglGetSwapIntervalEXT',
    'wglGetSyncValuesOML',
    'wglGetVideoDeviceNV',
    'wglGetVideoInfoNV',
    'wglIsEnabledFrameLockI3D',
    'wglIsEnabledGenlockI3D',
    'wglJoinSwapGroupNV',
    'wglLoadDisplayColorTableEXT',
    'wglMakeContextCurrentARB',
    'wglMakeContextCurrentEXT',
    'wglQueryCurrentContextNV',
    'wglQueryFrameCountNV',
    'wglQueryFrameLockMasterI3D',
    'wglQueryFrameTrackingI3D',
    'wglQueryGenlockMaxSourceDelayI3D',
    'wglQueryMaxSwapGroupsNV',
    'wglQueryPbufferARB',
    'wglQueryPbufferEXT',
    'wglQuerySwapGroupNV',
    'wglReleaseImageBufferEventsI3D',
    'wglReleasePbufferDCARB',
    'wglReleasePbufferDCEXT',
    'wglReleaseTexImageARB',
    'wglReleaseVideoDeviceNV',
    'wglReleaseVideoImageNV',
    'wglResetFrameCountNV',
    'wglRestoreBufferRegionARB',
    'wglSaveBufferRegionARB',
    'wglSendPbufferToVideoNV',
    'wglSetDigitalVideoParametersI3D',
    'wglSetGammaTableI3D',
    'wglSetGammaTableParametersI3D',
    'wglSetPbufferAttribARB',
    'wglSwapBuffersMscOML',
    'wglSwapIntervalEXT',
    'wglSwapLayerBuffersMscOML',
    'wglWaitForMscOML',
    'wglWaitForSbcOML',
]
# END GENERATED CONTENT (do not edit above this line)






#!/usr/bin/env python
from cparser import *

ctypes_type_map = {
     # typename signed  longs
    ('void',    True,   0): 'None',
    ('int',     True,   0): 'c_int',
    ('int',     False,  0): 'c_uint',
    ('int',     True,   1): 'c_long',
    ('int',     False,  1): 'c_ulong',
    ('int',     True,   2): 'c_longlong',
    ('int',     False,  2): 'c_ulonglong',
    ('char',    True,   0): 'c_char',
    ('char',    False,  0): 'c_ubyte',
    ('short',   True,   0): 'c_short',
    ('short',   False,  0): 'c_ushort',
    ('float',   True,   0): 'c_float',
    ('double',  True,   0): 'c_double',
    ('size_t',  True,   0): 'c_size_t',
    ('int8_t',  True,   0): 'c_int8',
    ('int16_t', True,   0): 'c_int16',
    ('int32_t', True,   0): 'c_int32',
    ('int64_t', True,   0): 'c_int64',
    ('uint8_t', True,   0): 'c_uint8',
    ('uint16_t',True,   0): 'c_uint16',
    ('uint32_t',True,   0): 'c_uint32',
    ('uint64_t',True,   0): 'c_uint64',
    ('wchar_t', True,   0): 'c_wchar',
    ('ptrdiff_t',True,  0): 'c_ptrdiff_t',  # Requires definition in preamble
}

reserved_names = ['None', 'True', 'False']

def get_ctypes_type(typ, declarator):
    signed = True
    typename = 'int'
    longs = 0
    t = None
    for specifier in typ.specifiers:
        if isinstance(specifier, StructTypeSpecifier):
            t = CtypesStruct(specifier)
        elif isinstance(specifier, EnumSpecifier):
            t = CtypesEnum(specifier)
        elif specifier == 'signed':
            signed = True
        elif specifier == 'unsigned':
            signed = False
        elif specifier == 'long':
            longs += 1
        else:
            typename = str(specifier)
    if not t:
        ctypes_name = ctypes_type_map.get((typename, signed, longs), typename)
        t = CtypesType(ctypes_name)

    while declarator and declarator.pointer:
        if declarator.parameters is not None:
            t = CtypesFunction(t, declarator.parameters)
        a = declarator.array
        while a:
            t = CtypesArray(t, a.size)
            a = a.array

        if type(t) == CtypesType and t.name == 'c_char':
            t = CtypesType('c_char_p')
        elif type(t) == CtypesType and t.name == 'c_wchar':
            t = CtypesType('c_wchar_p')
        else:
            t = CtypesPointer(t, declarator.qualifiers)
        declarator = declarator.pointer
    if declarator and declarator.parameters is not None:
        t = CtypesFunction(t, declarator.parameters)
    if declarator:
        a = declarator.array
        while a:
            t = CtypesArray(t, a.size)
            a = a.array
    return t
    
# Remove one level of indirection from function pointer; needed for typedefs
# and function parameters.
def remove_function_pointer(t):
    if type(t) == CtypesPointer and type(t.destination) == CtypesFunction:
        return t.destination
    elif type(t) == CtypesPointer:
        t.destination = remove_function_pointer(t.destination)
        return t
    else:
        return t

class CtypesTypeVisitor(object):
    def visit_struct(self, struct):
        pass

    def visit_enum(self, enum):
        pass

class CtypesType(object):
    def __init__(self, name):
        self.name = name

    def get_required_type_names(self):
        '''Return all type names defined or needed by this type'''
        return (self.name,)

    def visit(self, visitor):
        pass

    def __str__(self):
        return self.name

class CtypesPointer(CtypesType):
    def __init__(self, destination, qualifiers):
        self.destination = destination
        # ignore qualifiers, ctypes can't use them

    def get_required_type_names(self):
        if self.destination:
            return self.destination.get_required_type_names()
        else:
            return ()

    def visit(self, visitor):
        if self.destination:
            self.destination.visit(visitor)

    def __str__(self):
        return 'POINTER(%s)' % str(self.destination)

class CtypesArray(CtypesType):
    def __init__(self, base, count):
        self.base = base
        self.count = count
    
    def get_required_type_names(self):
        # XXX Could be sizeofs within count expression
        return self.base.get_required_type_names()
 
    def visit(self, visitor):
        # XXX Could be sizeofs within count expression
        self.base.visit(visitor)

    def __str__(self):
        if self.count is None:
            return 'POINTER(%s)' % str(self.base)
        if type(self.base) == CtypesArray:
            return '(%s) * %s' % (str(self.base), str(self.count))
        else:
            return '%s * %s' % (str(self.base), str(self.count))

class CtypesFunction(CtypesType):
    def __init__(self, restype, parameters):
        if parameters and parameters[-1] == '...':
            # XXX Hmm, how to handle VARARGS with ctypes?  For now,
            # drop it off (will cause errors).
            parameters = parameters[:-1]
            
        self.restype = restype

        # Don't allow POINTER(None) (c_void_p) as a restype... causes errors
        # when ctypes automagically returns it as an int.
        # Instead, convert to POINTER(c_void).  c_void is not a ctypes type,
        # you can make it any arbitrary type.
        if type(self.restype) == CtypesPointer and \
           type(self.restype.destination) == CtypesType and \
           self.restype.destination.name == 'None':
            self.restype = CtypesPointer(CtypesType('c_void'), ())

        self.argtypes = [remove_function_pointer(
                            get_ctypes_type(p.type, p.declarator)) \
                         for p in parameters]

    def get_required_type_names(self):
        lst = list(self.restype.get_required_type_names())
        for a in self.argtypes:
            lst += list(a.get_required_type_names())
        return lst

    def visit(self, visitor):
        self.restype.visit(visitor)
        for a in self.argtypes:
            a.visit(visitor)

    def __str__(self):
        return 'CFUNCTYPE(%s)' % ', '.join([str(self.restype)] + \
            [str(a) for a in self.argtypes])

last_tagnum = 0
def anonymous_struct_tag():
    global last_tagnum
    last_tagnum += 1
    return 'anon_%d' % last_tagnum

class CtypesStruct(CtypesType):
    def __init__(self, specifier):
        self.is_union = specifier.is_union
        self.tag = specifier.tag
        if not self.tag:
            self.tag = anonymous_struct_tag()

        if specifier.declarations:
            self.opaque = False
            self.members = []
            for declaration in specifier.declarations:
                t = get_ctypes_type(declaration.type, declaration.declarator)
                declarator = declaration.declarator
                if declarator is None:
                    # XXX TEMPORARY while struct with no typedef not filled in
                    return
                while declarator.pointer:
                    declarator = declarator.pointer
                name = declarator.identifier
                self.members.append((name, t))
        else:
            self.opaque = True
            self.members = []

    def get_required_type_names(self):
        lst = ['struct_%s' % self.tag]
        for m in self.members:
            lst += m[1].get_required_type_names()
        return lst

    def visit(self, visitor):
        visitor.visit_struct(self)

    def __str__(self):
        return 'struct_%s' % self.tag

last_tagnum = 0
def anonymous_enum_tag():
    global last_tagnum
    last_tagnum += 1
    return 'anon_%d' % last_tagnum

class CtypesEnum(CtypesType):
    def __init__(self, specifier):
        self.tag = specifier.tag
        if not self.tag:
            self.tag = anonymous_enum_tag()

        value = 0
        context = EvaluationContext()
        self.enumerators = []
        for e in specifier.enumerators:
            if e.expression:
                try:
                    value = int(e.expression.evaluate(context))
                except:
                    pass
            self.enumerators.append((e.name, value))
            value += 1

    def get_required_type_names(self):
        return []

    def visit(self, visitor):
        visitor.visit_enum(self)

    def __str__(self):
        return 'enum_%s' % self.tag

class CtypesParser(CParser):
    '''Parse a C file for declarations that can be used by ctypes.
    
    Subclass and override the handle_ctypes_* methods.
    '''
    def handle_define(self, name, value, filename, lineno):
        # Handle #define style of typedeffing.
        # XXX At the moment, just a hack for `int`, which is used by
        # Status and Bool in Xlib.h.  More complete functionality would
        # parse value as a type (back into cparser).
        if value == 'int':
            t = CtypesType('c_int')
            self.handle_ctypes_type_definition(
                name, t, filename, lineno)

    def handle_define_constant(self, name, value, filename, lineno):
        if name in reserved_names:
            name += '_'
        self.handle_ctypes_constant(name, value, filename, lineno)

    def handle_declaration(self, declaration, filename, lineno):
        t = get_ctypes_type(declaration.type, declaration.declarator)
        declarator = declaration.declarator
        if declarator is None:
            # XXX TEMPORARY while struct with no typedef not filled in
            return
        while declarator.pointer:
            declarator = declarator.pointer
        name = declarator.identifier
        if declaration.storage == 'typedef':
            self.handle_ctypes_type_definition(
                name, remove_function_pointer(t), filename, lineno)
        elif type(t) == CtypesFunction:
            self.handle_ctypes_function(
                name, t.restype, t.argtypes, filename, lineno)
        elif declaration.storage != 'static':
            self.handle_ctypes_variable(name, t, filename, lineno)

    # ctypes parser interface.  Override these methods in your subclass.

    def handle_ctypes_constant(self, name, value, filename, lineno):
        pass

    def handle_ctypes_type_definition(self, name, ctype, filename, lineno):
        pass

    def handle_ctypes_function(self, name, restype, argtypes, filename, lineno):
        pass

    def handle_ctypes_variable(self, name, ctype, filename, lineno):
        pass




import pytest
import time
import weakref

import pyglet
_debug = False
pyglet.options['debug_media'] = _debug
from pyglet.media.synthesis import Silence

try:
    from pyglet.media.drivers import openal
    import pyglet.media.drivers.openal.adaptation
except ImportError:
    openal = None


pytestmark = pytest.mark.skipif(openal is None, reason='No OpenAL available.')


def almost_equal(f1, f2, eps=0.0001):
    return abs(f1 - f2) < eps


def almost_equal_coords(c1, c2, eps=0.0001):
    return all(almost_equal(f1, f2, eps) for f1, f2 in zip(c1, c2))


@pytest.fixture
def device():
    device = openal.interface.OpenALDevice()
    yield device
    device.close()


def test_device_create_close(device):
    assert device.is_ready
    device.close()
    assert not device.is_ready


def test_device_version(device):
    major, minor = device.get_version()
    assert major > 0
    assert minor > 0


def test_device_extensions(device):
    extensions = device.get_extensions()
    assert len(extensions) > 0
    for ext in extensions:
        assert isinstance(ext, str)


def test_context_create_delete(device):
    context = device.create_context()
    assert context is not None
    context.delete()


@pytest.fixture
def context(device):
    context = device.create_context()
    yield context
    context.delete()


def test_context_make_current(context):
    context.make_current()


@pytest.fixture
def buffer_pool(context):
    return context.device.buffer_pool


@pytest.fixture
def buf(buffer_pool):
    buf = buffer_pool.get_buffer()
    yield buf
    buf.delete()


def test_buffer_create_delete(buf):
    assert buf.is_valid
    assert buf.al_name is not None
    assert buf.name > 0
    buf.delete()
    assert not buf.is_valid


def test_buffer_data(buf):
    assert buf.is_valid
    audio_source = Silence(1.)
    buf.data(audio_source.get_audio_data(audio_source.audio_format.bytes_per_second),
             audio_source.audio_format)
    assert buf.is_valid
    buf.delete()
    assert not buf.is_valid


def test_bufferpool_get_single_buffer(buffer_pool):
    assert len(buffer_pool) == 0

    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0


def test_bufferpool_return_valid_buffer(buffer_pool):
    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0

    buffer_pool.return_buffers((buf,))
    assert len(buffer_pool) == 1

    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0


def test_bufferpool_get_multiple_buffers(buffer_pool):
    bufs = buffer_pool.get_buffers(3)
    assert bufs is not None
    assert len(bufs) == 3
    for buf in bufs:
        assert buf.is_valid
    assert len(buffer_pool) == 0


def test_bufferpool_return_multiple_valid_buffers(buffer_pool):
    bufs = buffer_pool.get_buffers(3)
    assert bufs is not None
    assert len(bufs) == 3
    for buf in bufs:
        assert buf.is_valid
    assert len(buffer_pool) == 0

    return_count = 0
    for buf in bufs:
        buffer_pool.return_buffers((buf,))
        return_count += 1
        assert len(buffer_pool) == return_count

    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 2


def test_bufferpool_return_invalid_buffer(buffer_pool):
    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0

    buf.delete()
    assert not buf.is_valid
    buffer_pool.return_buffers((buf,))
    assert len(buffer_pool) == 0

    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0


def test_bufferpool_invalidate_buffer_in_pool(buffer_pool):
    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0

    buffer_pool.return_buffers((buf,))
    assert len(buffer_pool) == 1

    buf.delete()
    assert not buf.is_valid

    buf = buffer_pool.get_buffer()
    assert buf is not None
    assert buf.is_valid
    assert len(buffer_pool) == 0


def test_source_create_delete(context):
    source = context.create_source()
    assert source.is_initial
    assert not source.is_playing
    assert not source.is_paused
    assert not source.is_stopped
    assert source.buffers_processed == 0
    assert source.byte_offset == 0
    source.delete()


@pytest.fixture
def source(context):
    source = context.create_source()
    yield source
    source.delete()


@pytest.fixture
def filled_buffer(source):
    buf = source.get_buffer()
    assert buf.is_valid
    audio_source = Silence(1.)
    buf.data(audio_source.get_audio_data(audio_source.audio_format.bytes_per_second),
             audio_source.audio_format)
    yield buf
    source.buffer_pool.return_buffers((buf,))


def test_source_queue_play_unqueue(context, filled_buffer):
    source = context.create_source()

    source.queue_buffer(filled_buffer)
    assert source.is_initial
    assert not source.is_playing
    assert not source.is_paused
    assert not source.is_stopped
    assert source.buffers_processed == 0
    assert source.buffers_queued == 1
    assert source.byte_offset == 0

    source.play()
    assert not source.is_initial
    assert source.is_playing
    assert not source.is_paused
    assert not source.is_stopped
    assert source.byte_offset == 0

    end_time = time.time() + 1.5
    while time.time() < end_time:
        if source.byte_offset > 0:
            break
        time.sleep(.1)
    assert source.byte_offset > 0

    end_time = time.time() + 1.5
    while time.time() < end_time:
        if source.buffers_processed > 0:
            break
        time.sleep(.1)
    assert source.buffers_processed == 1

    processed = source.unqueue_buffers()
    assert processed == 1
    assert source.buffers_processed == 0
    assert source.buffers_queued == 0

    assert not source.is_initial
    assert not source.is_playing
    assert not source.is_paused
    assert source.is_stopped


@pytest.fixture
def filled_source(source, filled_buffer):
    source.queue_buffer(filled_buffer)
    return source


def test_source_pause_stop(filled_source):
    assert filled_source.is_initial
    assert not filled_source.is_playing
    assert not filled_source.is_paused
    assert not filled_source.is_stopped

    filled_source.play()
    assert not filled_source.is_initial
    assert filled_source.is_playing
    assert not filled_source.is_paused
    assert not filled_source.is_stopped

    filled_source.pause()
    assert not filled_source.is_initial
    assert not filled_source.is_playing
    assert filled_source.is_paused
    assert not filled_source.is_stopped

    filled_source.play()
    assert not filled_source.is_initial
    assert filled_source.is_playing
    assert not filled_source.is_paused
    assert not filled_source.is_stopped

    filled_source.stop()
    assert not filled_source.is_initial
    assert not filled_source.is_playing
    assert not filled_source.is_paused
    assert filled_source.is_stopped


def test_source_prop_position(filled_source):
    assert almost_equal_coords(filled_source.position, (0., 0., 0.))
    filled_source.position = 1., 2., 3.
    assert almost_equal_coords(filled_source.position, (1., 2., 3.))


def test_source_prop_velocity(filled_source):
    assert almost_equal_coords(filled_source.velocity, (0., 0., 0.))
    filled_source.velocity = 1., 2., 3.
    assert almost_equal_coords(filled_source.velocity, (1., 2., 3.))


def test_source_prop_gain(filled_source):
    assert almost_equal(filled_source.gain, 1.)
    filled_source.gain = 8.5
    assert almost_equal(filled_source.gain, 8.5)


def test_source_prop_min_gain(filled_source):
    assert almost_equal(filled_source.min_gain, 0.)
    filled_source.min_gain = .5
    assert almost_equal(filled_source.min_gain, .5)


def test_source_prop_max_gain(filled_source):
    assert almost_equal(filled_source.max_gain, 1.)
    filled_source.max_gain = .8
    assert almost_equal(filled_source.max_gain, .8)


def test_source_prop_reference_distance(filled_source):
    assert almost_equal(filled_source.reference_distance, 1.)
    filled_source.reference_distance = 10.3
    assert almost_equal(filled_source.reference_distance, 10.3)


def test_source_prop_rolloff_factor(filled_source):
    assert almost_equal(filled_source.rolloff_factor, 1.)
    filled_source.rolloff_factor = 4.5
    assert almost_equal(filled_source.rolloff_factor, 4.5)


def test_source_prop_max_distance(filled_source):
    assert filled_source.max_distance > 500.0  # No definition of MAX_FLOAT available, 1000.0 on OSX
    filled_source.max_distance = 500.
    assert almost_equal(filled_source.max_distance, 500.)


def test_source_prop_pitch(filled_source):
    assert almost_equal(filled_source.pitch, 1.)
    filled_source.pitch = 3.14
    assert almost_equal(filled_source.pitch, 3.14)


def test_source_prop_direction(filled_source):
    assert almost_equal_coords(filled_source.direction, (0., 0., 0.))
    filled_source.direction = 1., 2., 3.
    assert almost_equal_coords(filled_source.direction, (1., 2., 3.))


def test_source_prop_cone_inner_angle(filled_source):
    assert almost_equal(filled_source.cone_inner_angle, 360.)
    filled_source.cone_inner_angle = 180.
    assert almost_equal(filled_source.cone_inner_angle, 180.)


def test_source_prop_cone_outer_angle(filled_source):
    assert almost_equal(filled_source.cone_outer_angle, 360.)
    filled_source.cone_outer_angle = 90.
    assert almost_equal(filled_source.cone_outer_angle, 90.)


def test_source_prop_cone_outer_gain(filled_source):
    assert almost_equal(filled_source.cone_outer_gain, 0.)
    filled_source.cone_outer_gain = .6
    assert almost_equal(filled_source.cone_outer_gain, .6)


def test_source_prop_sec_offset(filled_source):
    assert almost_equal(filled_source.sec_offset, 0.)
    filled_source.play()
    filled_source.pause()
    filled_source.sec_offset = .1
    # Not stable: assert almost_equal(filled_source.sec_offset, .1)


def test_source_prop_sample_offset(filled_source):
    assert almost_equal(filled_source.sample_offset, 0.)
    filled_source.play()
    filled_source.pause()
    filled_source.sample_offset = 5.
    # Not stable: assert almost_equal(filled_source.sample_offset, 5.)


def test_source_prop_byte_offset(filled_source):
    assert almost_equal(filled_source.byte_offset, 0.)
    filled_source.play()
    filled_source.pause()
    filled_source.byte_offset = 8.
    # Not stable: assert almost_equal(filled_source.byte_offset, 8.)


def test_listener_prop_position(context):
    listener = openal.interface.OpenALListener()
    assert almost_equal_coords(listener.position, (0., 0., 0.))
    filled_source.position = 1., 2., 3.
    #TODO assert almost_equal_coords(listener.position, (1., 2., 3.))


def test_listener_prop_velocity(context):
    listener = openal.interface.OpenALListener()
    assert almost_equal_coords(listener.velocity, (0., 0., 0.))
    filled_source.velocity = 1., 2., 3.
    #TODO assert almost_equal_coords(listener.velocity, (1., 2., 3.))


def test_listener_prop_gain(context):
    listener = openal.interface.OpenALListener()
    assert almost_equal(listener.gain, 1.)
    filled_source.gain = 8.5
    #TODO assert almost_equal(listener.gain, 8.5)


def test_listener_prop_orientation(context):
    listener = openal.interface.OpenALListener()
    orientation = listener.orientation
    assert almost_equal_coords(orientation.at, (0., 0., -1.))
    assert almost_equal_coords(orientation.up, (0., 1., 0.))
    listener.orientation = ((1., 2., 3.), (4., 5., 6.))
    orientation = listener.orientation
    assert almost_equal_coords(orientation.at, (1., 2., 3.))
    assert almost_equal_coords(orientation.up, (4., 5., 6.))





from .adaptation import PulseAudioDriver

import pyglet

_debug = pyglet.options['debug_media']


def create_audio_driver():
    driver = PulseAudioDriver()
    driver.connect()
    if _debug:
        driver.dump_debug_info()
    return driver



from abc import ABCMeta, abstractmethod
from enum import Enum, auto
from typing import Dict, Optional

from pyglet import event


class DeviceState(Enum):
    ACTIVE = auto()
    DISABLED = auto()
    MISSING = auto()
    UNPLUGGED = auto()


class DeviceFlow(Enum):
    OUTPUT = auto()
    INPUT = auto()
    INPUT_OUTPUT = auto()


class AudioDevice:
    """Base class for a platform independent audio device.
       _platform_state and _platform_flow is used to make device state numbers."""
    platform_state: Dict[int, DeviceState] = {}  # Must be defined by the parent.
    platform_flow: Dict[int, DeviceFlow] = {}  # Must be defined by the parent.

    def __init__(self, dev_id: str, name: str, description: str, flow: int, state: int):
        self.id = dev_id
        self.flow = flow  # platform value
        self.state = state  # platform value
        self.name = name
        self.description = description

    def __repr__(self):
        return "{}(name='{}', state={}, flow={})".format(
            self.__class__.__name__, self.name, self.platform_state[self.state].name, self.platform_flow[self.flow].name)


class AbstractAudioDeviceManager(event.EventDispatcher, metaclass=ABCMeta):

    def __del__(self):
        """Required to remove handlers before exit, as it can cause problems with the event system's weakrefs."""
        self.remove_handlers(self)

    @abstractmethod
    def get_default_output(self):
        """Returns a default active output device or None if none available."""
        pass

    @abstractmethod
    def get_default_input(self):
        """Returns a default active input device or None if none available."""
        pass

    @abstractmethod
    def get_output_devices(self):
        """Returns a list of all active output devices."""
        pass

    @abstractmethod
    def get_input_devices(self):
        """Returns a list of all active input devices."""
        pass

    @abstractmethod
    def get_all_devices(self):
        """Returns a list of all audio devices, no matter what state they are in."""
        pass

    def on_device_state_changed(self, device: AudioDevice, old_state: DeviceState, new_state: DeviceState):
        """Event, occurs when the state of a device changes, provides the old state and new state."""
        pass

    def on_device_added(self, device: AudioDevice):
        """Event, occurs when a new device is added to the system."""
        pass

    def on_device_removed(self, device: AudioDevice):
        """Event, occurs when an existing device is removed from the system."""
        pass

    def on_default_changed(self, device: Optional[AudioDevice], flow: DeviceFlow):
        """Event, occurs when the default audio device changes.
           If there is no device that can be the default on the system, can be None.
           The flow determines whether an input or output device became it's respective default.
        """
        pass


AbstractAudioDeviceManager.register_event_type('on_device_state_changed')
AbstractAudioDeviceManager.register_event_type('on_device_added')
AbstractAudioDeviceManager.register_event_type('on_device_removed')
AbstractAudioDeviceManager.register_event_type('on_default_changed')



import ctypes

from .base import PlatformEventLoop

from pyglet.libs.win32 import _kernel32, _user32, types, constants
from pyglet.libs.win32.types import HANDLE, TIMECAPS


class Win32EventLoop(PlatformEventLoop):
    def __init__(self):
        super().__init__()

        self._next_idle_time = None

        # Force immediate creation of an event queue on this thread -- note
        # that since event loop is created on pyglet.app import, whatever
        # imports pyglet.app _must_ own the main run loop.
        msg = types.MSG()
        _user32.PeekMessageW(ctypes.byref(msg), 0,
                             constants.WM_USER, constants.WM_USER,
                             constants.PM_NOREMOVE)

        self._event_thread = _kernel32.GetCurrentThreadId()

        self._wait_objects = []
        self._recreate_wait_objects_array()

        self._timer_proc = types.TIMERPROC(self._timer_proc_func)
        self._timer = _user32.SetTimer(0, 0, constants.USER_TIMER_MAXIMUM, self._timer_proc)
        self._timer_func = None

        # Windows Multimedia timer precision functions
        # https://learn.microsoft.com/en-us/windows/win32/api/timeapi/nf-timeapi-timebeginperiod
        self._winmm = ctypes.windll.LoadLibrary('winmm')
        timecaps = TIMECAPS()
        self._winmm.timeGetDevCaps(ctypes.byref(timecaps), ctypes.sizeof(timecaps))
        self._timer_precision = min(max(1, timecaps.wPeriodMin), timecaps.wPeriodMax)

    def add_wait_object(self, obj, func):
        self._wait_objects.append((obj, func))
        self._recreate_wait_objects_array()

    def remove_wait_object(self, obj):
        for i, (_object, _) in enumerate(self._wait_objects):
            if obj == _object:
                del self._wait_objects[i]
                break
        self._recreate_wait_objects_array()

    def _recreate_wait_objects_array(self):
        if not self._wait_objects:
            self._wait_objects_n = 0
            self._wait_objects_array = None
            return

        self._wait_objects_n = len(self._wait_objects)
        self._wait_objects_array = (HANDLE * self._wait_objects_n)(*[o for o, f in self._wait_objects])

    def start(self):
        if _kernel32.GetCurrentThreadId() != self._event_thread:
            raise RuntimeError('EventLoop.run() must be called from the same ' +
                               'thread that imports pyglet.app')

        self._timer_func = None

        self._winmm.timeBeginPeriod(self._timer_precision)

    def step(self, timeout=None):
        self.dispatch_posted_events()

        msg = types.MSG()
        if timeout is None:
            timeout = constants.INFINITE
        else:
            timeout = int(timeout * 1000)  # milliseconds

        result = _user32.MsgWaitForMultipleObjects(
            self._wait_objects_n,
            self._wait_objects_array,
            False,
            timeout,
            constants.QS_ALLINPUT)
        result -= constants.WAIT_OBJECT_0

        if result == self._wait_objects_n:
            while _user32.PeekMessageW(ctypes.byref(msg),
                                       0, 0, 0, constants.PM_REMOVE):
                _user32.TranslateMessage(ctypes.byref(msg))
                _user32.DispatchMessageW(ctypes.byref(msg))
        elif 0 <= result < self._wait_objects_n:
            obj, func = self._wait_objects[result]
            func()

        # Return True if timeout was interrupted.
        return result <= self._wait_objects_n

    def stop(self):
        self._winmm.timeEndPeriod(self._timer_precision)

    def notify(self):
        # Nudge the event loop with a message it will discard.  Note that only
        # user events are actually posted.  The posted event will not
        # interrupt the window move/size drag loop -- it seems there's no way
        # to do this.
        _user32.PostThreadMessageW(self._event_thread, constants.WM_USER, 0, 0)

    def set_timer(self, func, interval):
        if func is None or interval is None:
            interval = constants.USER_TIMER_MAXIMUM
        else:
            interval = int(interval * 1000)  # milliseconds

        self._timer_func = func
        _user32.SetTimer(0, self._timer, interval, self._timer_proc)

    def _timer_proc_func(self, hwnd, msg, timer, t):
        if self._timer_func:
            self._timer_func()



"""Code by Richard Jones, released into the public domain.

Inspired by http://screamyguy.net/lines/index.htm

This code uses a single drawing buffer so that successive drawing passes
may be used to create the line fading effect. The fading is achieved
by drawing a translucent black quad over the entire scene before drawing
the next line segment.

Note: when working with a single buffer it is always a good idea to
glFlush() when you've finished your rendering.
"""


import sys
import random

import pyglet
from pyglet.gl import (
    glEnable,
    glBlendFunc,
    glFlush,
    Config,
    GL_BLEND,
    GL_LINE_SMOOTH,
    GL_LINES,
    GL_SRC_ALPHA,
    GL_ONE_MINUS_SRC_ALPHA,
)

# open a single-buffered window so we can do cheap accumulation
config = Config(double_buffer=False)
window = pyglet.window.Window(fullscreen='-fs' in sys.argv, config=config)


class Line:
    batch = pyglet.graphics.Batch()
    program = pyglet.shapes.get_default_shader()
    lines = program.vertex_list(100, GL_LINES, batch=batch, colors=('Bn', (255, 255, 255, 255) * 100))
    black = pyglet.shapes.Rectangle(0, 0, window.width, window.height, color=(0, 0, 0, 32), batch=batch)
    unallocated = list(range(100))
    active = []

    mouse_x = mouse_y = 0

    def __init__(self):
        self.n = self.unallocated.pop()
        self.active.append(self)
        self.x = self.lx = random.randint(0, window.width)
        self.y = self.ly = random.randint(0, window.height)
        self.dx = random.randint(-70, 70)
        self.dy = random.randint(-70, 70)
        self.damping = random.random() * .15 + .8
        self.power = random.random() * .1 + .05

    def update(self, dt):
        # calculate my acceleration based on the distance to the mouse
        # pointer and my acceleration power
        dx2 = (self.x - self.mouse_x) / self.power
        dy2 = (self.y - self.mouse_y) / self.power

        # now figure my new velocity
        self.dx -= dx2 * dt
        self.dy -= dy2 * dt
        self.dx *= self.damping
        self.dy *= self.damping

        # calculate new line endpoints
        self.lx = self.x
        self.ly = self.y
        self.x += self.dx * dt
        self.y += self.dy * dt

    @classmethod
    def on_draw(cls):
        cls.batch.draw()
        glFlush()

    @classmethod
    def on_mouse_motion(cls, x, y, dx, dy):
        cls.mouse_x, cls.mouse_y = x, y

    @classmethod
    def tick(cls, dt):
        if len(cls.active) < 50 and random.random() < .1:
            cls()

        if len(cls.active) > 10 and random.random() < .01:
            line = cls.active.pop(0)
            cls.unallocated.append(line.n)

        # update line positions
        for n, line in enumerate(cls.active):
            line.update(dt)
            cls.lines.position[n*4:n*4+4] = [line.lx, line.ly, line.x, line.y]


glEnable(GL_BLEND)
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
glEnable(GL_LINE_SMOOTH)


pyglet.clock.schedule_interval(Line.tick, 1/30)
window.push_handlers(Line)

pyglet.app.run()
